# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from typing import Dict, List
from typing_extensions import Literal, NotRequired, TypedDict


class InvoiceCreatePreviewParams(RequestOptions):
    automatic_tax: NotRequired["InvoiceCreatePreviewParamsAutomaticTax"]
    """
    Settings for automatic tax lookup for this invoice preview.
    """
    billing_cadence: NotRequired[str]
    """
    The identifier of the billing cadence for which you'd like to retrieve the upcoming invoice.Cannot be provided when `subscription`, `schedule`, `subscription_details` or `schedule_details` are provided.
    """
    currency: NotRequired[str]
    """
    The currency to preview this invoice in. Defaults to that of `customer` if not specified.
    """
    customer: NotRequired[str]
    """
    The identifier of the customer whose upcoming invoice you'd like to retrieve. If `automatic_tax` is enabled then one of `customer`, `customer_details`, `subscription`, or `schedule` must be set.
    """
    customer_account: NotRequired[str]
    """
    The identifier of the account whose upcoming invoice you'd like to retrieve. If `automatic_tax` is enabled then one of `customer`, `customer_account`, `customer_details`, `subscription`, or `schedule` must be set.
    """
    customer_details: NotRequired["InvoiceCreatePreviewParamsCustomerDetails"]
    """
    Details about the customer you want to invoice or overrides for an existing customer. If `automatic_tax` is enabled then one of `customer`, `customer_details`, `subscription`, or `schedule` must be set.
    """
    discounts: NotRequired[
        "Literal['']|List[InvoiceCreatePreviewParamsDiscount]"
    ]
    """
    The coupons to redeem into discounts for the invoice preview. If not specified, inherits the discount from the subscription or customer. This works for both coupons directly applied to an invoice and coupons applied to a subscription. Pass an empty string to avoid inheriting any discounts.
    """
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    invoice_items: NotRequired[List["InvoiceCreatePreviewParamsInvoiceItem"]]
    """
    List of invoice items to add or update in the upcoming invoice preview (up to 250).
    """
    issuer: NotRequired["InvoiceCreatePreviewParamsIssuer"]
    """
    The connected account that issues the invoice. The invoice is presented with the branding and support information of the specified account.
    """
    on_behalf_of: NotRequired["Literal['']|str"]
    """
    The account (if any) for which the funds of the invoice payment are intended. If set, the invoice will be presented with the branding and support information of the specified account. See the [Invoices with Connect](https://stripe.com/docs/billing/invoices/connect) documentation for details.
    """
    preview_mode: NotRequired[Literal["next", "recurring"]]
    """
    Customizes the types of values to include when calculating the invoice. Defaults to `next` if unspecified.
    """
    schedule: NotRequired[str]
    """
    The identifier of the schedule whose upcoming invoice you'd like to retrieve. Cannot be used with subscription or subscription fields.
    """
    schedule_details: NotRequired["InvoiceCreatePreviewParamsScheduleDetails"]
    """
    The schedule creation or modification params to apply as a preview. Cannot be used with `subscription` or `subscription_` prefixed fields.
    """
    subscription: NotRequired[str]
    """
    The identifier of the subscription for which you'd like to retrieve the upcoming invoice. If not provided, but a `subscription_details.items` is provided, you will preview creating a subscription with those items. If neither `subscription` nor `subscription_details.items` is provided, you will retrieve the next upcoming invoice from among the customer's subscriptions.
    """
    subscription_details: NotRequired[
        "InvoiceCreatePreviewParamsSubscriptionDetails"
    ]
    """
    The subscription creation or modification params to apply as a preview. Cannot be used with `schedule` or `schedule_details` fields.
    """


class InvoiceCreatePreviewParamsAutomaticTax(TypedDict):
    enabled: bool
    """
    Whether Stripe automatically computes tax on this invoice. Note that incompatible invoice items (invoice items with manually specified [tax rates](https://stripe.com/docs/api/tax_rates), negative amounts, or `tax_behavior=unspecified`) cannot be added to automatic tax invoices.
    """
    liability: NotRequired["InvoiceCreatePreviewParamsAutomaticTaxLiability"]
    """
    The account that's liable for tax. If set, the business address and tax registrations required to perform the tax calculation are loaded from this account. The tax transaction is returned in the report of the connected account.
    """


class InvoiceCreatePreviewParamsAutomaticTaxLiability(TypedDict):
    account: NotRequired[str]
    """
    The connected account being referenced when `type` is `account`.
    """
    type: Literal["account", "self"]
    """
    Type of the account referenced in the request.
    """


class InvoiceCreatePreviewParamsCustomerDetails(TypedDict):
    address: NotRequired[
        "Literal['']|InvoiceCreatePreviewParamsCustomerDetailsAddress"
    ]
    """
    The customer's address.
    """
    shipping: NotRequired[
        "Literal['']|InvoiceCreatePreviewParamsCustomerDetailsShipping"
    ]
    """
    The customer's shipping information. Appears on invoices emailed to this customer.
    """
    tax: NotRequired["InvoiceCreatePreviewParamsCustomerDetailsTax"]
    """
    Tax details about the customer.
    """
    tax_exempt: NotRequired["Literal['']|Literal['exempt', 'none', 'reverse']"]
    """
    The customer's tax exemption. One of `none`, `exempt`, or `reverse`.
    """
    tax_ids: NotRequired[
        List["InvoiceCreatePreviewParamsCustomerDetailsTaxId"]
    ]
    """
    The customer's tax IDs.
    """


class InvoiceCreatePreviewParamsCustomerDetailsAddress(TypedDict):
    city: NotRequired[str]
    """
    City, district, suburb, town, or village.
    """
    country: NotRequired[str]
    """
    Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
    """
    line1: NotRequired[str]
    """
    Address line 1, such as the street, PO Box, or company name.
    """
    line2: NotRequired[str]
    """
    Address line 2, such as the apartment, suite, unit, or building.
    """
    postal_code: NotRequired[str]
    """
    ZIP or postal code.
    """
    state: NotRequired[str]
    """
    State, county, province, or region.
    """


class InvoiceCreatePreviewParamsCustomerDetailsShipping(TypedDict):
    address: "InvoiceCreatePreviewParamsCustomerDetailsShippingAddress"
    """
    Customer shipping address.
    """
    name: str
    """
    Customer name.
    """
    phone: NotRequired[str]
    """
    Customer phone (including extension).
    """


class InvoiceCreatePreviewParamsCustomerDetailsShippingAddress(TypedDict):
    city: NotRequired[str]
    """
    City, district, suburb, town, or village.
    """
    country: NotRequired[str]
    """
    A freeform text field for the country. However, in order to activate some tax features, the format should be a two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
    """
    line1: NotRequired[str]
    """
    Address line 1, such as the street, PO Box, or company name.
    """
    line2: NotRequired[str]
    """
    Address line 2, such as the apartment, suite, unit, or building.
    """
    postal_code: NotRequired[str]
    """
    ZIP or postal code.
    """
    state: NotRequired[str]
    """
    State, county, province, or region.
    """


class InvoiceCreatePreviewParamsCustomerDetailsTax(TypedDict):
    ip_address: NotRequired["Literal['']|str"]
    """
    A recent IP address of the customer used for tax reporting and tax location inference. Stripe recommends updating the IP address when a new PaymentMethod is attached or the address field on the customer is updated. We recommend against updating this field more frequently since it could result in unexpected tax location/reporting outcomes.
    """


class InvoiceCreatePreviewParamsCustomerDetailsTaxId(TypedDict):
    type: Literal[
        "ad_nrt",
        "ae_trn",
        "al_tin",
        "am_tin",
        "ao_tin",
        "ar_cuit",
        "au_abn",
        "au_arn",
        "aw_tin",
        "az_tin",
        "ba_tin",
        "bb_tin",
        "bd_bin",
        "bf_ifu",
        "bg_uic",
        "bh_vat",
        "bj_ifu",
        "bo_tin",
        "br_cnpj",
        "br_cpf",
        "bs_tin",
        "by_tin",
        "ca_bn",
        "ca_gst_hst",
        "ca_pst_bc",
        "ca_pst_mb",
        "ca_pst_sk",
        "ca_qst",
        "cd_nif",
        "ch_uid",
        "ch_vat",
        "cl_tin",
        "cm_niu",
        "cn_tin",
        "co_nit",
        "cr_tin",
        "cv_nif",
        "de_stn",
        "do_rcn",
        "ec_ruc",
        "eg_tin",
        "es_cif",
        "et_tin",
        "eu_oss_vat",
        "eu_vat",
        "gb_vat",
        "ge_vat",
        "gn_nif",
        "hk_br",
        "hr_oib",
        "hu_tin",
        "id_npwp",
        "il_vat",
        "in_gst",
        "is_vat",
        "jp_cn",
        "jp_rn",
        "jp_trn",
        "ke_pin",
        "kg_tin",
        "kh_tin",
        "kr_brn",
        "kz_bin",
        "la_tin",
        "li_uid",
        "li_vat",
        "ma_vat",
        "md_vat",
        "me_pib",
        "mk_vat",
        "mr_nif",
        "mx_rfc",
        "my_frp",
        "my_itn",
        "my_sst",
        "ng_tin",
        "no_vat",
        "no_voec",
        "np_pan",
        "nz_gst",
        "om_vat",
        "pe_ruc",
        "ph_tin",
        "ro_tin",
        "rs_pib",
        "ru_inn",
        "ru_kpp",
        "sa_vat",
        "sg_gst",
        "sg_uen",
        "si_tin",
        "sn_ninea",
        "sr_fin",
        "sv_nit",
        "th_vat",
        "tj_tin",
        "tr_tin",
        "tw_vat",
        "tz_vat",
        "ua_vat",
        "ug_tin",
        "us_ein",
        "uy_ruc",
        "uz_tin",
        "uz_vat",
        "ve_rif",
        "vn_tin",
        "za_vat",
        "zm_tin",
        "zw_tin",
    ]
    """
    Type of the tax ID, one of `ad_nrt`, `ae_trn`, `al_tin`, `am_tin`, `ao_tin`, `ar_cuit`, `au_abn`, `au_arn`, `aw_tin`, `az_tin`, `ba_tin`, `bb_tin`, `bd_bin`, `bf_ifu`, `bg_uic`, `bh_vat`, `bj_ifu`, `bo_tin`, `br_cnpj`, `br_cpf`, `bs_tin`, `by_tin`, `ca_bn`, `ca_gst_hst`, `ca_pst_bc`, `ca_pst_mb`, `ca_pst_sk`, `ca_qst`, `cd_nif`, `ch_uid`, `ch_vat`, `cl_tin`, `cm_niu`, `cn_tin`, `co_nit`, `cr_tin`, `cv_nif`, `de_stn`, `do_rcn`, `ec_ruc`, `eg_tin`, `es_cif`, `et_tin`, `eu_oss_vat`, `eu_vat`, `gb_vat`, `ge_vat`, `gn_nif`, `hk_br`, `hr_oib`, `hu_tin`, `id_npwp`, `il_vat`, `in_gst`, `is_vat`, `jp_cn`, `jp_rn`, `jp_trn`, `ke_pin`, `kg_tin`, `kh_tin`, `kr_brn`, `kz_bin`, `la_tin`, `li_uid`, `li_vat`, `ma_vat`, `md_vat`, `me_pib`, `mk_vat`, `mr_nif`, `mx_rfc`, `my_frp`, `my_itn`, `my_sst`, `ng_tin`, `no_vat`, `no_voec`, `np_pan`, `nz_gst`, `om_vat`, `pe_ruc`, `ph_tin`, `ro_tin`, `rs_pib`, `ru_inn`, `ru_kpp`, `sa_vat`, `sg_gst`, `sg_uen`, `si_tin`, `sn_ninea`, `sr_fin`, `sv_nit`, `th_vat`, `tj_tin`, `tr_tin`, `tw_vat`, `tz_vat`, `ua_vat`, `ug_tin`, `us_ein`, `uy_ruc`, `uz_tin`, `uz_vat`, `ve_rif`, `vn_tin`, `za_vat`, `zm_tin`, or `zw_tin`
    """
    value: str
    """
    Value of the tax ID.
    """


class InvoiceCreatePreviewParamsDiscount(TypedDict):
    coupon: NotRequired[str]
    """
    ID of the coupon to create a new discount for.
    """
    discount: NotRequired[str]
    """
    ID of an existing discount on the object (or one of its ancestors) to reuse.
    """
    discount_end: NotRequired["InvoiceCreatePreviewParamsDiscountDiscountEnd"]
    """
    Details to determine how long the discount should be applied for.
    """
    promotion_code: NotRequired[str]
    """
    ID of the promotion code to create a new discount for.
    """


class InvoiceCreatePreviewParamsDiscountDiscountEnd(TypedDict):
    duration: NotRequired[
        "InvoiceCreatePreviewParamsDiscountDiscountEndDuration"
    ]
    """
    Time span for the redeemed discount.
    """
    timestamp: NotRequired[int]
    """
    A precise Unix timestamp for the discount to end. Must be in the future.
    """
    type: Literal["duration", "timestamp"]
    """
    The type of calculation made to determine when the discount ends.
    """


class InvoiceCreatePreviewParamsDiscountDiscountEndDuration(TypedDict):
    interval: Literal["day", "month", "week", "year"]
    """
    Specifies a type of interval unit. Either `day`, `week`, `month` or `year`.
    """
    interval_count: int
    """
    The number of intervals, as an whole number greater than 0. Stripe multiplies this by the interval type to get the overall duration.
    """


class InvoiceCreatePreviewParamsInvoiceItem(TypedDict):
    amount: NotRequired[int]
    """
    The integer amount in cents (or local equivalent) of previewed invoice item.
    """
    currency: NotRequired[str]
    """
    Three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html), in lowercase. Must be a [supported currency](https://stripe.com/docs/currencies). Only applicable to new invoice items.
    """
    description: NotRequired[str]
    """
    An arbitrary string which you can attach to the invoice item. The description is displayed in the invoice for easy tracking.
    """
    discountable: NotRequired[bool]
    """
    Explicitly controls whether discounts apply to this invoice item. Defaults to true, except for negative invoice items.
    """
    discounts: NotRequired[
        "Literal['']|List[InvoiceCreatePreviewParamsInvoiceItemDiscount]"
    ]
    """
    The coupons to redeem into discounts for the invoice item in the preview.
    """
    invoiceitem: NotRequired[str]
    """
    The ID of the invoice item to update in preview. If not specified, a new invoice item will be added to the preview of the upcoming invoice.
    """
    metadata: NotRequired["Literal['']|Dict[str, str]"]
    """
    Set of [key-value pairs](https://stripe.com/docs/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format. Individual keys can be unset by posting an empty value to them. All keys can be unset by posting an empty value to `metadata`.
    """
    period: NotRequired["InvoiceCreatePreviewParamsInvoiceItemPeriod"]
    """
    The period associated with this invoice item. When set to different values, the period will be rendered on the invoice. If you have [Stripe Revenue Recognition](https://stripe.com/docs/revenue-recognition) enabled, the period will be used to recognize and defer revenue. See the [Revenue Recognition documentation](https://stripe.com/docs/revenue-recognition/methodology/subscriptions-and-invoicing) for details.
    """
    price: NotRequired[str]
    """
    The ID of the price object. One of `price` or `price_data` is required.
    """
    price_data: NotRequired["InvoiceCreatePreviewParamsInvoiceItemPriceData"]
    """
    Data used to generate a new [Price](https://stripe.com/docs/api/prices) object inline. One of `price` or `price_data` is required.
    """
    quantity: NotRequired[int]
    """
    Non-negative integer. The quantity of units for the invoice item.
    """
    tax_behavior: NotRequired[Literal["exclusive", "inclusive", "unspecified"]]
    """
    Only required if a [default tax behavior](https://stripe.com/docs/tax/products-prices-tax-categories-tax-behavior#setting-a-default-tax-behavior-(recommended)) was not provided in the Stripe Tax settings. Specifies whether the price is considered inclusive of taxes or exclusive of taxes. One of `inclusive`, `exclusive`, or `unspecified`. Once specified as either `inclusive` or `exclusive`, it cannot be changed.
    """
    tax_code: NotRequired["Literal['']|str"]
    """
    A [tax code](https://stripe.com/docs/tax/tax-categories) ID.
    """
    tax_rates: NotRequired["Literal['']|List[str]"]
    """
    The tax rates that apply to the item. When set, any `default_tax_rates` do not apply to this item.
    """
    unit_amount: NotRequired[int]
    """
    The integer unit amount in cents (or local equivalent) of the charge to be applied to the upcoming invoice. This unit_amount will be multiplied by the quantity to get the full amount. If you want to apply a credit to the customer's account, pass a negative unit_amount.
    """
    unit_amount_decimal: NotRequired[str]
    """
    Same as `unit_amount`, but accepts a decimal value in cents (or local equivalent) with at most 12 decimal places. Only one of `unit_amount` and `unit_amount_decimal` can be set.
    """


class InvoiceCreatePreviewParamsInvoiceItemDiscount(TypedDict):
    coupon: NotRequired[str]
    """
    ID of the coupon to create a new discount for.
    """
    discount: NotRequired[str]
    """
    ID of an existing discount on the object (or one of its ancestors) to reuse.
    """
    discount_end: NotRequired[
        "InvoiceCreatePreviewParamsInvoiceItemDiscountDiscountEnd"
    ]
    """
    Details to determine how long the discount should be applied for.
    """
    promotion_code: NotRequired[str]
    """
    ID of the promotion code to create a new discount for.
    """


class InvoiceCreatePreviewParamsInvoiceItemDiscountDiscountEnd(TypedDict):
    duration: NotRequired[
        "InvoiceCreatePreviewParamsInvoiceItemDiscountDiscountEndDuration"
    ]
    """
    Time span for the redeemed discount.
    """
    timestamp: NotRequired[int]
    """
    A precise Unix timestamp for the discount to end. Must be in the future.
    """
    type: Literal["duration", "timestamp"]
    """
    The type of calculation made to determine when the discount ends.
    """


class InvoiceCreatePreviewParamsInvoiceItemDiscountDiscountEndDuration(
    TypedDict,
):
    interval: Literal["day", "month", "week", "year"]
    """
    Specifies a type of interval unit. Either `day`, `week`, `month` or `year`.
    """
    interval_count: int
    """
    The number of intervals, as an whole number greater than 0. Stripe multiplies this by the interval type to get the overall duration.
    """


class InvoiceCreatePreviewParamsInvoiceItemPeriod(TypedDict):
    end: int
    """
    The end of the period, which must be greater than or equal to the start. This value is inclusive.
    """
    start: int
    """
    The start of the period. This value is inclusive.
    """


class InvoiceCreatePreviewParamsInvoiceItemPriceData(TypedDict):
    currency: str
    """
    Three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html), in lowercase. Must be a [supported currency](https://stripe.com/docs/currencies).
    """
    product: str
    """
    The ID of the [Product](https://docs.stripe.com/api/products) that this [Price](https://docs.stripe.com/api/prices) will belong to.
    """
    tax_behavior: NotRequired[Literal["exclusive", "inclusive", "unspecified"]]
    """
    Only required if a [default tax behavior](https://stripe.com/docs/tax/products-prices-tax-categories-tax-behavior#setting-a-default-tax-behavior-(recommended)) was not provided in the Stripe Tax settings. Specifies whether the price is considered inclusive of taxes or exclusive of taxes. One of `inclusive`, `exclusive`, or `unspecified`. Once specified as either `inclusive` or `exclusive`, it cannot be changed.
    """
    unit_amount: NotRequired[int]
    """
    A positive integer in cents (or local equivalent) (or 0 for a free price) representing how much to charge.
    """
    unit_amount_decimal: NotRequired[str]
    """
    Same as `unit_amount`, but accepts a decimal value in cents (or local equivalent) with at most 12 decimal places. Only one of `unit_amount` and `unit_amount_decimal` can be set.
    """


class InvoiceCreatePreviewParamsIssuer(TypedDict):
    account: NotRequired[str]
    """
    The connected account being referenced when `type` is `account`.
    """
    type: Literal["account", "self"]
    """
    Type of the account referenced in the request.
    """


class InvoiceCreatePreviewParamsScheduleDetails(TypedDict):
    amendments: NotRequired[
        List["InvoiceCreatePreviewParamsScheduleDetailsAmendment"]
    ]
    """
    Changes to apply to the phases of the subscription schedule, in the order provided.
    """
    billing_behavior: NotRequired[
        Literal["prorate_on_next_phase", "prorate_up_front"]
    ]
    """
    Configures when the subscription schedule generates prorations for phase transitions. Possible values are `prorate_on_next_phase` or `prorate_up_front` with the default being `prorate_on_next_phase`. `prorate_on_next_phase` will apply phase changes and generate prorations at transition time. `prorate_up_front` will bill for all phases within the current billing cycle up front.
    """
    billing_mode: NotRequired[
        "InvoiceCreatePreviewParamsScheduleDetailsBillingMode"
    ]
    """
    Controls how prorations and invoices for subscriptions are calculated and orchestrated.
    """
    end_behavior: NotRequired[Literal["cancel", "release"]]
    """
    Behavior of the subscription schedule and underlying subscription when it ends. Possible values are `release` or `cancel` with the default being `release`. `release` will end the subscription schedule and keep the underlying subscription running. `cancel` will end the subscription schedule and cancel the underlying subscription.
    """
    phases: NotRequired[List["InvoiceCreatePreviewParamsScheduleDetailsPhase"]]
    """
    List representing phases of the subscription schedule. Each phase can be customized to have different durations, plans, and coupons. If there are multiple phases, the `end_date` of one phase will always equal the `start_date` of the next phase.
    """
    prebilling: NotRequired[
        "Literal['']|List[InvoiceCreatePreviewParamsScheduleDetailsPrebilling]"
    ]
    """
    Provide any time periods to bill in advance.
    """
    proration_behavior: NotRequired[
        Literal["always_invoice", "create_prorations", "none"]
    ]
    """
    In cases where the `schedule_details` params update the currently active phase, specifies if and how to prorate at the time of the request.
    """
    billing_schedules: NotRequired[
        List["InvoiceCreatePreviewParamsScheduleDetailsBillingSchedule"]
    ]
    """
    Sets the billing schedules for the subscription schedule.
    """


class InvoiceCreatePreviewParamsScheduleDetailsAmendment(TypedDict):
    amendment_end: NotRequired[
        "InvoiceCreatePreviewParamsScheduleDetailsAmendmentAmendmentEnd"
    ]
    """
    Details to identify the end of the time range modified by the proposed change. If not supplied, the amendment is considered a point-in-time operation that only affects the exact timestamp at `amendment_start`, and a restricted set of attributes is supported on the amendment.
    """
    amendment_start: (
        "InvoiceCreatePreviewParamsScheduleDetailsAmendmentAmendmentStart"
    )
    """
    Details to identify the earliest timestamp where the proposed change should take effect.
    """
    billing_cycle_anchor: NotRequired[Literal["amendment_start", "automatic"]]
    """
    For point-in-time amendments (having no `amendment_end`), this attribute lets you set or remove whether the subscription's billing cycle anchor is reset at the `amendment_start` timestamp.For time-span based amendments (having both `amendment_start` and `amendment_end`), the only value valid is `automatic`, which removes any previously configured billing cycle anchor resets scheduled to occur during the window of time spanned by the amendment.
    """
    discount_actions: NotRequired[
        List[
            "InvoiceCreatePreviewParamsScheduleDetailsAmendmentDiscountAction"
        ]
    ]
    """
    Changes to the coupons being redeemed or discounts being applied during the amendment time span.
    """
    item_actions: NotRequired[
        List["InvoiceCreatePreviewParamsScheduleDetailsAmendmentItemAction"]
    ]
    """
    Changes to the subscription items during the amendment time span.
    """
    metadata_actions: NotRequired[
        List[
            "InvoiceCreatePreviewParamsScheduleDetailsAmendmentMetadataAction"
        ]
    ]
    """
    Instructions for how to modify phase metadata
    """
    proration_behavior: NotRequired[
        Literal["always_invoice", "create_prorations", "none"]
    ]
    """
    Changes to how Stripe handles prorations during the amendment time span. Affects if and how prorations are created when a future phase starts. In cases where the amendment changes the currently active phase, it is used to determine whether or how to prorate now, at the time of the request. Also supported as a point-in-time operation when `amendment_end` is `null`.
    """
    set_pause_collection: NotRequired[
        "InvoiceCreatePreviewParamsScheduleDetailsAmendmentSetPauseCollection"
    ]
    """
    Defines how to pause collection for the underlying subscription throughout the duration of the amendment.
    """
    set_schedule_end: NotRequired[Literal["amendment_end", "amendment_start"]]
    """
    Ends the subscription schedule early as dictated by either the accompanying amendment's start or end.
    """
    trial_settings: NotRequired[
        "InvoiceCreatePreviewParamsScheduleDetailsAmendmentTrialSettings"
    ]
    """
    Settings related to subscription trials.
    """
    billing_schedules_actions: NotRequired[
        List[
            "InvoiceCreatePreviewParamsScheduleDetailsAmendmentBillingSchedulesAction"
        ]
    ]
    """
    Actions to apply to the billing schedules.
    """


class InvoiceCreatePreviewParamsScheduleDetailsAmendmentAmendmentEnd(
    TypedDict
):
    discount_end: NotRequired[
        "InvoiceCreatePreviewParamsScheduleDetailsAmendmentAmendmentEndDiscountEnd"
    ]
    """
    Use the `end` time of a given discount.
    """
    duration: NotRequired[
        "InvoiceCreatePreviewParamsScheduleDetailsAmendmentAmendmentEndDuration"
    ]
    """
    Time span for the amendment starting from the `amendment_start`.
    """
    timestamp: NotRequired[int]
    """
    A precise Unix timestamp for the amendment to end. Must be after the `amendment_start`.
    """
    type: Literal[
        "discount_end",
        "duration",
        "schedule_end",
        "timestamp",
        "trial_end",
        "trial_start",
        "upcoming_invoice",
    ]
    """
    Select one of three ways to pass the `amendment_end`.
    """


class InvoiceCreatePreviewParamsScheduleDetailsAmendmentAmendmentEndDiscountEnd(
    TypedDict,
):
    discount: str
    """
    The ID of a specific discount.
    """


class InvoiceCreatePreviewParamsScheduleDetailsAmendmentAmendmentEndDuration(
    TypedDict,
):
    interval: Literal["day", "month", "week", "year"]
    """
    Specifies a type of interval unit. Either `day`, `week`, `month` or `year`.
    """
    interval_count: int
    """
    The number of intervals, as an whole number greater than 0. Stripe multiplies this by the interval type to get the overall duration.
    """


class InvoiceCreatePreviewParamsScheduleDetailsAmendmentAmendmentStart(
    TypedDict,
):
    amendment_end: NotRequired[
        "InvoiceCreatePreviewParamsScheduleDetailsAmendmentAmendmentStartAmendmentEnd"
    ]
    """
    Details of another amendment in the same array, immediately after which this amendment should begin.
    """
    discount_end: NotRequired[
        "InvoiceCreatePreviewParamsScheduleDetailsAmendmentAmendmentStartDiscountEnd"
    ]
    """
    Use the `end` time of a given discount.
    """
    timestamp: NotRequired[int]
    """
    A precise Unix timestamp for the amendment to start.
    """
    type: Literal[
        "amendment_end",
        "discount_end",
        "now",
        "schedule_end",
        "timestamp",
        "trial_end",
        "trial_start",
        "upcoming_invoice",
    ]
    """
    Select one of three ways to pass the `amendment_start`.
    """


class InvoiceCreatePreviewParamsScheduleDetailsAmendmentAmendmentStartAmendmentEnd(
    TypedDict,
):
    index: int
    """
    The position of the previous amendment in the `amendments` array after which this amendment should begin. Indexes start from 0 and must be less than the index of the current amendment in the array.
    """


class InvoiceCreatePreviewParamsScheduleDetailsAmendmentAmendmentStartDiscountEnd(
    TypedDict,
):
    discount: str
    """
    The ID of a specific discount.
    """


class InvoiceCreatePreviewParamsScheduleDetailsAmendmentDiscountAction(
    TypedDict,
):
    add: NotRequired[
        "InvoiceCreatePreviewParamsScheduleDetailsAmendmentDiscountActionAdd"
    ]
    """
    Details of the discount to add.
    """
    remove: NotRequired[
        "InvoiceCreatePreviewParamsScheduleDetailsAmendmentDiscountActionRemove"
    ]
    """
    Details of the discount to remove.
    """
    set: NotRequired[
        "InvoiceCreatePreviewParamsScheduleDetailsAmendmentDiscountActionSet"
    ]
    """
    Details of the discount to replace the existing discounts with.
    """
    type: Literal["add", "remove", "set"]
    """
    Determines the type of discount action.
    """


class InvoiceCreatePreviewParamsScheduleDetailsAmendmentDiscountActionAdd(
    TypedDict,
):
    coupon: NotRequired[str]
    """
    The coupon code to redeem.
    """
    discount: NotRequired[str]
    """
    An ID of an existing discount for a coupon that was already redeemed.
    """
    discount_end: NotRequired[
        "InvoiceCreatePreviewParamsScheduleDetailsAmendmentDiscountActionAddDiscountEnd"
    ]
    """
    Details to determine how long the discount should be applied for.
    """
    index: NotRequired[int]
    """
    The index, starting at 0, at which to position the new discount. When not supplied, Stripe defaults to appending the discount to the end of the `discounts` array.
    """
    promotion_code: NotRequired[str]
    """
    The promotion code to redeem.
    """


class InvoiceCreatePreviewParamsScheduleDetailsAmendmentDiscountActionAddDiscountEnd(
    TypedDict,
):
    type: Literal["amendment_end"]
    """
    The type of calculation made to determine when the discount ends.
    """


class InvoiceCreatePreviewParamsScheduleDetailsAmendmentDiscountActionRemove(
    TypedDict,
):
    coupon: NotRequired[str]
    """
    The coupon code to remove from the `discounts` array.
    """
    discount: NotRequired[str]
    """
    The ID of a discount to remove from the `discounts` array.
    """
    promotion_code: NotRequired[str]
    """
    The ID of a promotion code to remove from the `discounts` array.
    """


class InvoiceCreatePreviewParamsScheduleDetailsAmendmentDiscountActionSet(
    TypedDict,
):
    coupon: NotRequired[str]
    """
    The coupon code to replace the `discounts` array with.
    """
    discount: NotRequired[str]
    """
    An ID of an existing discount to replace the `discounts` array with.
    """
    promotion_code: NotRequired[str]
    """
    An ID of an existing promotion code to replace the `discounts` array with.
    """


class InvoiceCreatePreviewParamsScheduleDetailsAmendmentItemAction(TypedDict):
    add: NotRequired[
        "InvoiceCreatePreviewParamsScheduleDetailsAmendmentItemActionAdd"
    ]
    """
    Details of the subscription item to add. If an item with the same `price` exists, it will be replaced by this new item. Otherwise, it adds the new item.
    """
    remove: NotRequired[
        "InvoiceCreatePreviewParamsScheduleDetailsAmendmentItemActionRemove"
    ]
    """
    Details of the subscription item to remove.
    """
    set: NotRequired[
        "InvoiceCreatePreviewParamsScheduleDetailsAmendmentItemActionSet"
    ]
    """
    Details of the subscription item to replace the existing items with. If an item with the `set[price]` already exists, the `items` array is not cleared. Instead, all of the other `set` properties that are passed in this request will replace the existing values for the configuration item.
    """
    type: Literal["add", "remove", "set"]
    """
    Determines the type of item action.
    """


class InvoiceCreatePreviewParamsScheduleDetailsAmendmentItemActionAdd(
    TypedDict,
):
    discounts: NotRequired[
        List[
            "InvoiceCreatePreviewParamsScheduleDetailsAmendmentItemActionAddDiscount"
        ]
    ]
    """
    The discounts applied to the item. Subscription item discounts are applied before subscription discounts.
    """
    metadata: NotRequired[Dict[str, str]]
    """
    Set of [key-value pairs](https://stripe.com/docs/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format. Individual keys can be unset by posting an empty value to them. All keys can be unset by posting an empty value to `metadata`.
    """
    price: str
    """
    The ID of the price object.
    """
    quantity: NotRequired[int]
    """
    Quantity for this item.
    """
    tax_rates: NotRequired[List[str]]
    """
    The tax rates that apply to this subscription item. When set, the `default_tax_rates` on the subscription do not apply to this `subscription_item`.
    """
    trial: NotRequired[
        "InvoiceCreatePreviewParamsScheduleDetailsAmendmentItemActionAddTrial"
    ]
    """
    Options that configure the trial on the subscription item.
    """


class InvoiceCreatePreviewParamsScheduleDetailsAmendmentItemActionAddDiscount(
    TypedDict,
):
    coupon: NotRequired[str]
    """
    ID of the coupon to create a new discount for.
    """
    discount: NotRequired[str]
    """
    ID of an existing discount on the object (or one of its ancestors) to reuse.
    """
    discount_end: NotRequired[
        "InvoiceCreatePreviewParamsScheduleDetailsAmendmentItemActionAddDiscountDiscountEnd"
    ]
    """
    Details to determine how long the discount should be applied for.
    """
    promotion_code: NotRequired[str]
    """
    ID of the promotion code to create a new discount for.
    """


class InvoiceCreatePreviewParamsScheduleDetailsAmendmentItemActionAddDiscountDiscountEnd(
    TypedDict,
):
    duration: NotRequired[
        "InvoiceCreatePreviewParamsScheduleDetailsAmendmentItemActionAddDiscountDiscountEndDuration"
    ]
    """
    Time span for the redeemed discount.
    """
    timestamp: NotRequired[int]
    """
    A precise Unix timestamp for the discount to end. Must be in the future.
    """
    type: Literal["duration", "timestamp"]
    """
    The type of calculation made to determine when the discount ends.
    """


class InvoiceCreatePreviewParamsScheduleDetailsAmendmentItemActionAddDiscountDiscountEndDuration(
    TypedDict,
):
    interval: Literal["day", "month", "week", "year"]
    """
    Specifies a type of interval unit. Either `day`, `week`, `month` or `year`.
    """
    interval_count: int
    """
    The number of intervals, as an whole number greater than 0. Stripe multiplies this by the interval type to get the overall duration.
    """


class InvoiceCreatePreviewParamsScheduleDetailsAmendmentItemActionAddTrial(
    TypedDict,
):
    converts_to: NotRequired[List[str]]
    """
    List of price IDs which, if present on the subscription following a paid trial, constitute opting-in to the paid trial. Currently only supports at most 1 price ID.
    """
    type: Literal["free", "paid"]
    """
    Determines the type of trial for this item.
    """


class InvoiceCreatePreviewParamsScheduleDetailsAmendmentItemActionRemove(
    TypedDict,
):
    price: str
    """
    ID of a price to remove.
    """


class InvoiceCreatePreviewParamsScheduleDetailsAmendmentItemActionSet(
    TypedDict,
):
    discounts: NotRequired[
        List[
            "InvoiceCreatePreviewParamsScheduleDetailsAmendmentItemActionSetDiscount"
        ]
    ]
    """
    If an item with the `price` already exists, passing this will override the `discounts` array on the subscription item that matches that price. Otherwise, the `items` array is cleared and a single new item is added with the supplied `discounts`.
    """
    metadata: NotRequired[Dict[str, str]]
    """
    If an item with the `price` already exists, passing this will override the `metadata` on the subscription item that matches that price. Otherwise, the `items` array is cleared and a single new item is added with the supplied `metadata`.
    """
    price: str
    """
    The ID of the price object.
    """
    quantity: NotRequired[int]
    """
    If an item with the `price` already exists, passing this will override the quantity on the subscription item that matches that price. Otherwise, the `items` array is cleared and a single new item is added with the supplied `quantity`.
    """
    tax_rates: NotRequired[List[str]]
    """
    If an item with the `price` already exists, passing this will override the `tax_rates` array on the subscription item that matches that price. Otherwise, the `items` array is cleared and a single new item is added with the supplied `tax_rates`.
    """
    trial: NotRequired[
        "InvoiceCreatePreviewParamsScheduleDetailsAmendmentItemActionSetTrial"
    ]
    """
    If an item with the `price` already exists, passing this will override the `trial` configuration on the subscription item that matches that price. Otherwise, the `items` array is cleared and a single new item is added with the supplied `trial`.
    """


class InvoiceCreatePreviewParamsScheduleDetailsAmendmentItemActionSetDiscount(
    TypedDict,
):
    coupon: NotRequired[str]
    """
    ID of the coupon to create a new discount for.
    """
    discount: NotRequired[str]
    """
    ID of an existing discount on the object (or one of its ancestors) to reuse.
    """
    discount_end: NotRequired[
        "InvoiceCreatePreviewParamsScheduleDetailsAmendmentItemActionSetDiscountDiscountEnd"
    ]
    """
    Details to determine how long the discount should be applied for.
    """
    promotion_code: NotRequired[str]
    """
    ID of the promotion code to create a new discount for.
    """


class InvoiceCreatePreviewParamsScheduleDetailsAmendmentItemActionSetDiscountDiscountEnd(
    TypedDict,
):
    duration: NotRequired[
        "InvoiceCreatePreviewParamsScheduleDetailsAmendmentItemActionSetDiscountDiscountEndDuration"
    ]
    """
    Time span for the redeemed discount.
    """
    timestamp: NotRequired[int]
    """
    A precise Unix timestamp for the discount to end. Must be in the future.
    """
    type: Literal["duration", "timestamp"]
    """
    The type of calculation made to determine when the discount ends.
    """


class InvoiceCreatePreviewParamsScheduleDetailsAmendmentItemActionSetDiscountDiscountEndDuration(
    TypedDict,
):
    interval: Literal["day", "month", "week", "year"]
    """
    Specifies a type of interval unit. Either `day`, `week`, `month` or `year`.
    """
    interval_count: int
    """
    The number of intervals, as an whole number greater than 0. Stripe multiplies this by the interval type to get the overall duration.
    """


class InvoiceCreatePreviewParamsScheduleDetailsAmendmentItemActionSetTrial(
    TypedDict,
):
    converts_to: NotRequired[List[str]]
    """
    List of price IDs which, if present on the subscription following a paid trial, constitute opting-in to the paid trial. Currently only supports at most 1 price ID.
    """
    type: Literal["free", "paid"]
    """
    Determines the type of trial for this item.
    """


class InvoiceCreatePreviewParamsScheduleDetailsAmendmentMetadataAction(
    TypedDict,
):
    add: NotRequired[Dict[str, str]]
    """
    Key-value pairs to add to schedule phase metadata. These values will merge with existing schedule phase metadata.
    """
    remove: NotRequired[List[str]]
    """
    Keys to remove from schedule phase metadata.
    """
    set: NotRequired["Literal['']|Dict[str, str]"]
    """
    Key-value pairs to set as schedule phase metadata. Existing schedule phase metadata will be overwritten.
    """
    type: Literal["add", "remove", "set"]
    """
    Select one of three ways to update phase-level `metadata` on subscription schedules.
    """


class InvoiceCreatePreviewParamsScheduleDetailsAmendmentSetPauseCollection(
    TypedDict,
):
    set: NotRequired[
        "InvoiceCreatePreviewParamsScheduleDetailsAmendmentSetPauseCollectionSet"
    ]
    """
    Details of the pause_collection behavior to apply to the amendment.
    """
    type: Literal["remove", "set"]
    """
    Determines the type of the pause_collection amendment.
    """


class InvoiceCreatePreviewParamsScheduleDetailsAmendmentSetPauseCollectionSet(
    TypedDict,
):
    behavior: Literal["keep_as_draft", "mark_uncollectible", "void"]
    """
    The payment collection behavior for this subscription while paused. One of `keep_as_draft`, `mark_uncollectible`, or `void`.
    """


class InvoiceCreatePreviewParamsScheduleDetailsAmendmentTrialSettings(
    TypedDict,
):
    end_behavior: NotRequired[
        "InvoiceCreatePreviewParamsScheduleDetailsAmendmentTrialSettingsEndBehavior"
    ]
    """
    Defines how the subscription should behave when a trial ends.
    """


class InvoiceCreatePreviewParamsScheduleDetailsAmendmentTrialSettingsEndBehavior(
    TypedDict,
):
    prorate_up_front: NotRequired[Literal["defer", "include"]]
    """
    Configure how an opt-in following a paid trial is billed when using `billing_behavior: prorate_up_front`.
    """


class InvoiceCreatePreviewParamsScheduleDetailsAmendmentBillingSchedulesAction(
    TypedDict,
):
    applies_to: NotRequired[
        List[
            "InvoiceCreatePreviewParamsScheduleDetailsAmendmentBillingSchedulesActionAppliesTo"
        ]
    ]
    """
    Specify which subscription items the billing schedule applies to.
    """
    type: Literal["remove", "set"]
    """
    Select the action.
    """


class InvoiceCreatePreviewParamsScheduleDetailsAmendmentBillingSchedulesActionAppliesTo(
    TypedDict,
):
    price: NotRequired[str]
    """
    The ID of the price object.
    """
    type: Literal["price"]
    """
    Controls which subscription items the billing schedule applies to.
    """


class InvoiceCreatePreviewParamsScheduleDetailsBillingMode(TypedDict):
    flexible: NotRequired[
        "InvoiceCreatePreviewParamsScheduleDetailsBillingModeFlexible"
    ]
    """
    Configure behavior for flexible billing mode.
    """
    type: Literal["classic", "flexible"]
    """
    Controls the calculation and orchestration of prorations and invoices for subscriptions. If no value is passed, the default is `flexible`.
    """


class InvoiceCreatePreviewParamsScheduleDetailsBillingModeFlexible(TypedDict):
    proration_discounts: NotRequired[Literal["included", "itemized"]]
    """
    Controls how invoices and invoice items display proration amounts and discount amounts.
    """


class InvoiceCreatePreviewParamsScheduleDetailsPhase(TypedDict):
    add_invoice_items: NotRequired[
        List["InvoiceCreatePreviewParamsScheduleDetailsPhaseAddInvoiceItem"]
    ]
    """
    A list of prices and quantities that will generate invoice items appended to the next invoice for this phase. You may pass up to 20 items.
    """
    application_fee_percent: NotRequired[float]
    """
    A non-negative decimal between 0 and 100, with at most two decimal places. This represents the percentage of the subscription invoice total that will be transferred to the application owner's Stripe account. The request must be made by a platform account on a connected account in order to set an application fee percentage. For more information, see the application fees [documentation](https://stripe.com/docs/connect/subscriptions#collecting-fees-on-subscriptions).
    """
    automatic_tax: NotRequired[
        "InvoiceCreatePreviewParamsScheduleDetailsPhaseAutomaticTax"
    ]
    """
    Automatic tax settings for this phase.
    """
    billing_cycle_anchor: NotRequired[Literal["automatic", "phase_start"]]
    """
    Can be set to `phase_start` to set the anchor to the start of the phase or `automatic` to automatically change it if needed. Cannot be set to `phase_start` if this phase specifies a trial. For more information, see the billing cycle [documentation](https://stripe.com/docs/billing/subscriptions/billing-cycle).
    """
    billing_thresholds: NotRequired[
        "Literal['']|InvoiceCreatePreviewParamsScheduleDetailsPhaseBillingThresholds"
    ]
    """
    Define thresholds at which an invoice will be sent, and the subscription advanced to a new billing period. Pass an empty string to remove previously-defined thresholds.
    """
    collection_method: NotRequired[
        Literal["charge_automatically", "send_invoice"]
    ]
    """
    Either `charge_automatically`, or `send_invoice`. When charging automatically, Stripe will attempt to pay the underlying subscription at the end of each billing cycle using the default source attached to the customer. When sending an invoice, Stripe will email your customer an invoice with payment instructions and mark the subscription as `active`. Defaults to `charge_automatically` on creation.
    """
    currency: NotRequired[str]
    """
    Three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html), in lowercase. Must be a [supported currency](https://stripe.com/docs/currencies).
    """
    default_payment_method: NotRequired[str]
    """
    ID of the default payment method for the subscription schedule. It must belong to the customer associated with the subscription schedule. If not set, invoices will use the default payment method in the customer's invoice settings.
    """
    default_tax_rates: NotRequired["Literal['']|List[str]"]
    """
    A list of [Tax Rate](https://stripe.com/docs/api/tax_rates) ids. These Tax Rates will set the Subscription's [`default_tax_rates`](https://stripe.com/docs/api/subscriptions/create#create_subscription-default_tax_rates), which means they will be the Invoice's [`default_tax_rates`](https://stripe.com/docs/api/invoices/create#create_invoice-default_tax_rates) for any Invoices issued by the Subscription during this Phase.
    """
    description: NotRequired["Literal['']|str"]
    """
    Subscription description, meant to be displayable to the customer. Use this field to optionally store an explanation of the subscription for rendering in Stripe surfaces and certain local payment methods UIs.
    """
    discounts: NotRequired[
        "Literal['']|List[InvoiceCreatePreviewParamsScheduleDetailsPhaseDiscount]"
    ]
    """
    The coupons to redeem into discounts for the schedule phase. If not specified, inherits the discount from the subscription's customer. Pass an empty string to avoid inheriting any discounts.
    """
    duration: NotRequired[
        "InvoiceCreatePreviewParamsScheduleDetailsPhaseDuration"
    ]
    """
    The number of intervals the phase should last. If set, `end_date` must not be set.
    """
    end_date: NotRequired["int|Literal['now']"]
    """
    The date at which this phase of the subscription schedule ends. If set, `iterations` must not be set.
    """
    invoice_settings: NotRequired[
        "InvoiceCreatePreviewParamsScheduleDetailsPhaseInvoiceSettings"
    ]
    """
    All invoices will be billed using the specified settings.
    """
    items: List["InvoiceCreatePreviewParamsScheduleDetailsPhaseItem"]
    """
    List of configuration items, each with an attached price, to apply during this phase of the subscription schedule.
    """
    metadata: NotRequired[Dict[str, str]]
    """
    Set of [key-value pairs](https://stripe.com/docs/api/metadata) that you can attach to a phase. Metadata on a schedule's phase will update the underlying subscription's `metadata` when the phase is entered, adding new keys and replacing existing keys in the subscription's `metadata`. Individual keys in the subscription's `metadata` can be unset by posting an empty value to them in the phase's `metadata`. To unset all keys in the subscription's `metadata`, update the subscription directly or unset every key individually from the phase's `metadata`.
    """
    on_behalf_of: NotRequired[str]
    """
    The account on behalf of which to charge, for each of the associated subscription's invoices.
    """
    pause_collection: NotRequired[
        "InvoiceCreatePreviewParamsScheduleDetailsPhasePauseCollection"
    ]
    """
    If specified, payment collection for this subscription will be paused. Note that the subscription status will be unchanged and will not be updated to `paused`. Learn more about [pausing collection](https://stripe.com/docs/billing/subscriptions/pause-payment).
    """
    proration_behavior: NotRequired[
        Literal["always_invoice", "create_prorations", "none"]
    ]
    """
    Controls whether the subscription schedule should create [prorations](https://stripe.com/docs/billing/subscriptions/prorations) when transitioning to this phase if there is a difference in billing configuration. It's different from the request-level [proration_behavior](https://stripe.com/docs/api/subscription_schedules/update#update_subscription_schedule-proration_behavior) parameter which controls what happens if the update request affects the billing configuration (item price, quantity, etc.) of the current phase.
    """
    start_date: NotRequired["int|Literal['now']"]
    """
    The date at which this phase of the subscription schedule starts or `now`. Must be set on the first phase.
    """
    transfer_data: NotRequired[
        "InvoiceCreatePreviewParamsScheduleDetailsPhaseTransferData"
    ]
    """
    The data with which to automatically create a Transfer for each of the associated subscription's invoices.
    """
    trial: NotRequired[bool]
    """
    If set to true the entire phase is counted as a trial and the customer will not be charged for any fees.
    """
    trial_continuation: NotRequired[Literal["continue", "none"]]
    """
    Specify trial behavior when crossing phase boundaries
    """
    trial_end: NotRequired["int|Literal['now']"]
    """
    Sets the phase to trialing from the start date to this date. Must be before the phase end date, can not be combined with `trial`
    """
    trial_settings: NotRequired[
        "InvoiceCreatePreviewParamsScheduleDetailsPhaseTrialSettings"
    ]
    """
    Settings related to subscription trials.
    """


class InvoiceCreatePreviewParamsScheduleDetailsPhaseAddInvoiceItem(TypedDict):
    discounts: NotRequired[
        List[
            "InvoiceCreatePreviewParamsScheduleDetailsPhaseAddInvoiceItemDiscount"
        ]
    ]
    """
    The coupons to redeem into discounts for the item.
    """
    metadata: NotRequired[Dict[str, str]]
    """
    Set of [key-value pairs](https://stripe.com/docs/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format. Individual keys can be unset by posting an empty value to them. All keys can be unset by posting an empty value to `metadata`.
    """
    period: NotRequired[
        "InvoiceCreatePreviewParamsScheduleDetailsPhaseAddInvoiceItemPeriod"
    ]
    """
    The period associated with this invoice item. If not set, `period.start.type` defaults to `max_item_period_start` and `period.end.type` defaults to `min_item_period_end`.
    """
    price: NotRequired[str]
    """
    The ID of the price object. One of `price` or `price_data` is required.
    """
    price_data: NotRequired[
        "InvoiceCreatePreviewParamsScheduleDetailsPhaseAddInvoiceItemPriceData"
    ]
    """
    Data used to generate a new [Price](https://stripe.com/docs/api/prices) object inline. One of `price` or `price_data` is required.
    """
    quantity: NotRequired[int]
    """
    Quantity for this item. Defaults to 1.
    """
    tax_rates: NotRequired["Literal['']|List[str]"]
    """
    The tax rates which apply to the item. When set, the `default_tax_rates` do not apply to this item.
    """


class InvoiceCreatePreviewParamsScheduleDetailsPhaseAddInvoiceItemDiscount(
    TypedDict,
):
    coupon: NotRequired[str]
    """
    ID of the coupon to create a new discount for.
    """
    discount: NotRequired[str]
    """
    ID of an existing discount on the object (or one of its ancestors) to reuse.
    """
    discount_end: NotRequired[
        "InvoiceCreatePreviewParamsScheduleDetailsPhaseAddInvoiceItemDiscountDiscountEnd"
    ]
    """
    Details to determine how long the discount should be applied for.
    """
    promotion_code: NotRequired[str]
    """
    ID of the promotion code to create a new discount for.
    """


class InvoiceCreatePreviewParamsScheduleDetailsPhaseAddInvoiceItemDiscountDiscountEnd(
    TypedDict,
):
    duration: NotRequired[
        "InvoiceCreatePreviewParamsScheduleDetailsPhaseAddInvoiceItemDiscountDiscountEndDuration"
    ]
    """
    Time span for the redeemed discount.
    """
    timestamp: NotRequired[int]
    """
    A precise Unix timestamp for the discount to end. Must be in the future.
    """
    type: Literal["duration", "timestamp"]
    """
    The type of calculation made to determine when the discount ends.
    """


class InvoiceCreatePreviewParamsScheduleDetailsPhaseAddInvoiceItemDiscountDiscountEndDuration(
    TypedDict,
):
    interval: Literal["day", "month", "week", "year"]
    """
    Specifies a type of interval unit. Either `day`, `week`, `month` or `year`.
    """
    interval_count: int
    """
    The number of intervals, as an whole number greater than 0. Stripe multiplies this by the interval type to get the overall duration.
    """


class InvoiceCreatePreviewParamsScheduleDetailsPhaseAddInvoiceItemPeriod(
    TypedDict,
):
    end: (
        "InvoiceCreatePreviewParamsScheduleDetailsPhaseAddInvoiceItemPeriodEnd"
    )
    """
    End of the invoice item period.
    """
    start: "InvoiceCreatePreviewParamsScheduleDetailsPhaseAddInvoiceItemPeriodStart"
    """
    Start of the invoice item period.
    """


class InvoiceCreatePreviewParamsScheduleDetailsPhaseAddInvoiceItemPeriodEnd(
    TypedDict,
):
    timestamp: NotRequired[int]
    """
    A precise Unix timestamp for the end of the invoice item period. Must be greater than or equal to `period.start`.
    """
    type: Literal["min_item_period_end", "phase_end", "timestamp"]
    """
    Select how to calculate the end of the invoice item period.
    """


class InvoiceCreatePreviewParamsScheduleDetailsPhaseAddInvoiceItemPeriodStart(
    TypedDict,
):
    timestamp: NotRequired[int]
    """
    A precise Unix timestamp for the start of the invoice item period. Must be less than or equal to `period.end`.
    """
    type: Literal["max_item_period_start", "phase_start", "timestamp"]
    """
    Select how to calculate the start of the invoice item period.
    """


class InvoiceCreatePreviewParamsScheduleDetailsPhaseAddInvoiceItemPriceData(
    TypedDict,
):
    currency: str
    """
    Three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html), in lowercase. Must be a [supported currency](https://stripe.com/docs/currencies).
    """
    product: str
    """
    The ID of the [Product](https://docs.stripe.com/api/products) that this [Price](https://docs.stripe.com/api/prices) will belong to.
    """
    tax_behavior: NotRequired[Literal["exclusive", "inclusive", "unspecified"]]
    """
    Only required if a [default tax behavior](https://stripe.com/docs/tax/products-prices-tax-categories-tax-behavior#setting-a-default-tax-behavior-(recommended)) was not provided in the Stripe Tax settings. Specifies whether the price is considered inclusive of taxes or exclusive of taxes. One of `inclusive`, `exclusive`, or `unspecified`. Once specified as either `inclusive` or `exclusive`, it cannot be changed.
    """
    unit_amount: NotRequired[int]
    """
    A positive integer in cents (or local equivalent) (or 0 for a free price) representing how much to charge or a negative integer representing the amount to credit to the customer.
    """
    unit_amount_decimal: NotRequired[str]
    """
    Same as `unit_amount`, but accepts a decimal value in cents (or local equivalent) with at most 12 decimal places. Only one of `unit_amount` and `unit_amount_decimal` can be set.
    """


class InvoiceCreatePreviewParamsScheduleDetailsPhaseAutomaticTax(TypedDict):
    enabled: bool
    """
    Enabled automatic tax calculation which will automatically compute tax rates on all invoices generated by the subscription.
    """
    liability: NotRequired[
        "InvoiceCreatePreviewParamsScheduleDetailsPhaseAutomaticTaxLiability"
    ]
    """
    The account that's liable for tax. If set, the business address and tax registrations required to perform the tax calculation are loaded from this account. The tax transaction is returned in the report of the connected account.
    """


class InvoiceCreatePreviewParamsScheduleDetailsPhaseAutomaticTaxLiability(
    TypedDict,
):
    account: NotRequired[str]
    """
    The connected account being referenced when `type` is `account`.
    """
    type: Literal["account", "self"]
    """
    Type of the account referenced in the request.
    """


class InvoiceCreatePreviewParamsScheduleDetailsPhaseBillingThresholds(
    TypedDict,
):
    amount_gte: NotRequired[int]
    """
    Monetary threshold that triggers the subscription to advance to a new billing period
    """
    reset_billing_cycle_anchor: NotRequired[bool]
    """
    Indicates if the `billing_cycle_anchor` should be reset when a threshold is reached. If true, `billing_cycle_anchor` will be updated to the date/time the threshold was last reached; otherwise, the value will remain unchanged.
    """


class InvoiceCreatePreviewParamsScheduleDetailsPhaseDiscount(TypedDict):
    coupon: NotRequired[str]
    """
    ID of the coupon to create a new discount for.
    """
    discount: NotRequired[str]
    """
    ID of an existing discount on the object (or one of its ancestors) to reuse.
    """
    discount_end: NotRequired[
        "InvoiceCreatePreviewParamsScheduleDetailsPhaseDiscountDiscountEnd"
    ]
    """
    Details to determine how long the discount should be applied for.
    """
    promotion_code: NotRequired[str]
    """
    ID of the promotion code to create a new discount for.
    """


class InvoiceCreatePreviewParamsScheduleDetailsPhaseDiscountDiscountEnd(
    TypedDict,
):
    duration: NotRequired[
        "InvoiceCreatePreviewParamsScheduleDetailsPhaseDiscountDiscountEndDuration"
    ]
    """
    Time span for the redeemed discount.
    """
    timestamp: NotRequired[int]
    """
    A precise Unix timestamp for the discount to end. Must be in the future.
    """
    type: Literal["duration", "timestamp"]
    """
    The type of calculation made to determine when the discount ends.
    """


class InvoiceCreatePreviewParamsScheduleDetailsPhaseDiscountDiscountEndDuration(
    TypedDict,
):
    interval: Literal["day", "month", "week", "year"]
    """
    Specifies a type of interval unit. Either `day`, `week`, `month` or `year`.
    """
    interval_count: int
    """
    The number of intervals, as an whole number greater than 0. Stripe multiplies this by the interval type to get the overall duration.
    """


class InvoiceCreatePreviewParamsScheduleDetailsPhaseDuration(TypedDict):
    interval: Literal["day", "month", "week", "year"]
    """
    Specifies phase duration. Either `day`, `week`, `month` or `year`.
    """
    interval_count: NotRequired[int]
    """
    The multiplier applied to the interval.
    """


class InvoiceCreatePreviewParamsScheduleDetailsPhaseInvoiceSettings(TypedDict):
    account_tax_ids: NotRequired["Literal['']|List[str]"]
    """
    The account tax IDs associated with this phase of the subscription schedule. Will be set on invoices generated by this phase of the subscription schedule.
    """
    days_until_due: NotRequired[int]
    """
    Number of days within which a customer must pay invoices generated by this subscription schedule. This value will be `null` for subscription schedules where `billing=charge_automatically`.
    """
    issuer: NotRequired[
        "InvoiceCreatePreviewParamsScheduleDetailsPhaseInvoiceSettingsIssuer"
    ]
    """
    The connected account that issues the invoice. The invoice is presented with the branding and support information of the specified account.
    """


class InvoiceCreatePreviewParamsScheduleDetailsPhaseInvoiceSettingsIssuer(
    TypedDict,
):
    account: NotRequired[str]
    """
    The connected account being referenced when `type` is `account`.
    """
    type: Literal["account", "self"]
    """
    Type of the account referenced in the request.
    """


class InvoiceCreatePreviewParamsScheduleDetailsPhaseItem(TypedDict):
    billing_thresholds: NotRequired[
        "Literal['']|InvoiceCreatePreviewParamsScheduleDetailsPhaseItemBillingThresholds"
    ]
    """
    Define thresholds at which an invoice will be sent, and the subscription advanced to a new billing period. Pass an empty string to remove previously-defined thresholds.
    """
    discounts: NotRequired[
        "Literal['']|List[InvoiceCreatePreviewParamsScheduleDetailsPhaseItemDiscount]"
    ]
    """
    The coupons to redeem into discounts for the subscription item.
    """
    metadata: NotRequired[Dict[str, str]]
    """
    Set of [key-value pairs](https://stripe.com/docs/api/metadata) that you can attach to a configuration item. Metadata on a configuration item will update the underlying subscription item's `metadata` when the phase is entered, adding new keys and replacing existing keys. Individual keys in the subscription item's `metadata` can be unset by posting an empty value to them in the configuration item's `metadata`. To unset all keys in the subscription item's `metadata`, update the subscription item directly or unset every key individually from the configuration item's `metadata`.
    """
    plan: NotRequired[str]
    """
    The plan ID to subscribe to. You may specify the same ID in `plan` and `price`.
    """
    price: NotRequired[str]
    """
    The ID of the price object.
    """
    price_data: NotRequired[
        "InvoiceCreatePreviewParamsScheduleDetailsPhaseItemPriceData"
    ]
    """
    Data used to generate a new [Price](https://stripe.com/docs/api/prices) object inline.
    """
    quantity: NotRequired[int]
    """
    Quantity for the given price. Can be set only if the price's `usage_type` is `licensed` and not `metered`.
    """
    tax_rates: NotRequired["Literal['']|List[str]"]
    """
    A list of [Tax Rate](https://stripe.com/docs/api/tax_rates) ids. These Tax Rates will override the [`default_tax_rates`](https://stripe.com/docs/api/subscriptions/create#create_subscription-default_tax_rates) on the Subscription. When updating, pass an empty string to remove previously-defined tax rates.
    """
    trial: NotRequired[
        "InvoiceCreatePreviewParamsScheduleDetailsPhaseItemTrial"
    ]
    """
    Options that configure the trial on the subscription item.
    """


class InvoiceCreatePreviewParamsScheduleDetailsPhaseItemBillingThresholds(
    TypedDict,
):
    usage_gte: int
    """
    Number of units that meets the billing threshold to advance the subscription to a new billing period (e.g., it takes 10 $5 units to meet a $50 [monetary threshold](https://stripe.com/docs/api/subscriptions/update#update_subscription-billing_thresholds-amount_gte))
    """


class InvoiceCreatePreviewParamsScheduleDetailsPhaseItemDiscount(TypedDict):
    coupon: NotRequired[str]
    """
    ID of the coupon to create a new discount for.
    """
    discount: NotRequired[str]
    """
    ID of an existing discount on the object (or one of its ancestors) to reuse.
    """
    discount_end: NotRequired[
        "InvoiceCreatePreviewParamsScheduleDetailsPhaseItemDiscountDiscountEnd"
    ]
    """
    Details to determine how long the discount should be applied for.
    """
    promotion_code: NotRequired[str]
    """
    ID of the promotion code to create a new discount for.
    """


class InvoiceCreatePreviewParamsScheduleDetailsPhaseItemDiscountDiscountEnd(
    TypedDict,
):
    duration: NotRequired[
        "InvoiceCreatePreviewParamsScheduleDetailsPhaseItemDiscountDiscountEndDuration"
    ]
    """
    Time span for the redeemed discount.
    """
    timestamp: NotRequired[int]
    """
    A precise Unix timestamp for the discount to end. Must be in the future.
    """
    type: Literal["duration", "timestamp"]
    """
    The type of calculation made to determine when the discount ends.
    """


class InvoiceCreatePreviewParamsScheduleDetailsPhaseItemDiscountDiscountEndDuration(
    TypedDict,
):
    interval: Literal["day", "month", "week", "year"]
    """
    Specifies a type of interval unit. Either `day`, `week`, `month` or `year`.
    """
    interval_count: int
    """
    The number of intervals, as an whole number greater than 0. Stripe multiplies this by the interval type to get the overall duration.
    """


class InvoiceCreatePreviewParamsScheduleDetailsPhaseItemPriceData(TypedDict):
    currency: str
    """
    Three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html), in lowercase. Must be a [supported currency](https://stripe.com/docs/currencies).
    """
    product: str
    """
    The ID of the [Product](https://docs.stripe.com/api/products) that this [Price](https://docs.stripe.com/api/prices) will belong to.
    """
    recurring: (
        "InvoiceCreatePreviewParamsScheduleDetailsPhaseItemPriceDataRecurring"
    )
    """
    The recurring components of a price such as `interval` and `interval_count`.
    """
    tax_behavior: NotRequired[Literal["exclusive", "inclusive", "unspecified"]]
    """
    Only required if a [default tax behavior](https://stripe.com/docs/tax/products-prices-tax-categories-tax-behavior#setting-a-default-tax-behavior-(recommended)) was not provided in the Stripe Tax settings. Specifies whether the price is considered inclusive of taxes or exclusive of taxes. One of `inclusive`, `exclusive`, or `unspecified`. Once specified as either `inclusive` or `exclusive`, it cannot be changed.
    """
    unit_amount: NotRequired[int]
    """
    A positive integer in cents (or local equivalent) (or 0 for a free price) representing how much to charge.
    """
    unit_amount_decimal: NotRequired[str]
    """
    Same as `unit_amount`, but accepts a decimal value in cents (or local equivalent) with at most 12 decimal places. Only one of `unit_amount` and `unit_amount_decimal` can be set.
    """


class InvoiceCreatePreviewParamsScheduleDetailsPhaseItemPriceDataRecurring(
    TypedDict,
):
    interval: Literal["day", "month", "week", "year"]
    """
    Specifies billing frequency. Either `day`, `week`, `month` or `year`.
    """
    interval_count: NotRequired[int]
    """
    The number of intervals between subscription billings. For example, `interval=month` and `interval_count=3` bills every 3 months. Maximum of three years interval allowed (3 years, 36 months, or 156 weeks).
    """


class InvoiceCreatePreviewParamsScheduleDetailsPhaseItemTrial(TypedDict):
    converts_to: NotRequired[List[str]]
    """
    List of price IDs which, if present on the subscription following a paid trial, constitute opting-in to the paid trial. Currently only supports at most 1 price ID.
    """
    type: Literal["free", "paid"]
    """
    Determines the type of trial for this item.
    """


class InvoiceCreatePreviewParamsScheduleDetailsPhasePauseCollection(TypedDict):
    behavior: Literal["keep_as_draft", "mark_uncollectible", "void"]
    """
    The payment collection behavior for this subscription while paused. One of `keep_as_draft`, `mark_uncollectible`, or `void`.
    """


class InvoiceCreatePreviewParamsScheduleDetailsPhaseTransferData(TypedDict):
    amount_percent: NotRequired[float]
    """
    A non-negative decimal between 0 and 100, with at most two decimal places. This represents the percentage of the subscription invoice total that will be transferred to the destination account. By default, the entire amount is transferred to the destination.
    """
    destination: str
    """
    ID of an existing, connected Stripe account.
    """


class InvoiceCreatePreviewParamsScheduleDetailsPhaseTrialSettings(TypedDict):
    end_behavior: NotRequired[
        "InvoiceCreatePreviewParamsScheduleDetailsPhaseTrialSettingsEndBehavior"
    ]
    """
    Defines how the subscription should behave when a trial ends.
    """


class InvoiceCreatePreviewParamsScheduleDetailsPhaseTrialSettingsEndBehavior(
    TypedDict,
):
    prorate_up_front: NotRequired[Literal["defer", "include"]]
    """
    Configure how an opt-in following a paid trial is billed when using `billing_behavior: prorate_up_front`.
    """


class InvoiceCreatePreviewParamsScheduleDetailsPrebilling(TypedDict):
    bill_until: NotRequired[
        "InvoiceCreatePreviewParamsScheduleDetailsPrebillingBillUntil"
    ]
    """
    The end of the prebilled time period.
    """
    iterations: NotRequired[int]
    """
    This is used to determine the number of billing cycles to prebill.
    """


class InvoiceCreatePreviewParamsScheduleDetailsPrebillingBillUntil(TypedDict):
    amendment_end: NotRequired[
        "InvoiceCreatePreviewParamsScheduleDetailsPrebillingBillUntilAmendmentEnd"
    ]
    """
    End the prebilled period when a specified amendment ends.
    """
    duration: NotRequired[
        "InvoiceCreatePreviewParamsScheduleDetailsPrebillingBillUntilDuration"
    ]
    """
    Time span for prebilling, starting from `bill_from`.
    """
    timestamp: NotRequired[int]
    """
    End the prebilled period at a precise integer timestamp, starting from the Unix epoch.
    """
    type: Literal["amendment_end", "duration", "schedule_end", "timestamp"]
    """
    Select one of several ways to pass the `bill_until` value.
    """


class InvoiceCreatePreviewParamsScheduleDetailsPrebillingBillUntilAmendmentEnd(
    TypedDict,
):
    index: int
    """
    The position of the amendment in the `amendments` array at which prebilling should end. Indexes start from 0 and must be less than the total number of supplied amendments.
    """


class InvoiceCreatePreviewParamsScheduleDetailsPrebillingBillUntilDuration(
    TypedDict,
):
    interval: Literal["day", "month", "week", "year"]
    """
    Specifies a type of interval unit. Either `day`, `week`, `month` or `year`.
    """
    interval_count: int
    """
    The number of intervals, as an whole number greater than 0. Stripe multiplies this by the interval type to get the overall duration.
    """


class InvoiceCreatePreviewParamsScheduleDetailsBillingSchedule(TypedDict):
    applies_to: NotRequired[
        List[
            "InvoiceCreatePreviewParamsScheduleDetailsBillingScheduleAppliesTo"
        ]
    ]
    """
    Configure billing schedule differently for individual subscription items.
    """
    bill_until: NotRequired[
        "InvoiceCreatePreviewParamsScheduleDetailsBillingScheduleBillUntil"
    ]
    """
    The end date for the billing schedule.
    """
    key: NotRequired[str]
    """
    Specify a key for the billing schedule. Must be unique to this field, alphanumeric, and up to 200 characters. If not provided, a unique key will be generated.
    """


class InvoiceCreatePreviewParamsScheduleDetailsBillingScheduleAppliesTo(
    TypedDict,
):
    price: NotRequired[str]
    """
    The ID of the price object.
    """
    type: Literal["price"]
    """
    Controls which subscription items the billing schedule applies to.
    """


class InvoiceCreatePreviewParamsScheduleDetailsBillingScheduleBillUntil(
    TypedDict,
):
    duration: NotRequired[
        "InvoiceCreatePreviewParamsScheduleDetailsBillingScheduleBillUntilDuration"
    ]
    """
    Specifies the billing period.
    """
    timestamp: NotRequired[int]
    """
    The end date of the billing schedule.
    """
    type: Literal[
        "amendment_end",
        "duration",
        "line_ends_at",
        "schedule_end",
        "timestamp",
        "upcoming_invoice",
    ]
    """
    Describes how the billing schedule will determine the end date. Either `duration` or `timestamp`.
    """


class InvoiceCreatePreviewParamsScheduleDetailsBillingScheduleBillUntilDuration(
    TypedDict,
):
    interval: Literal["day", "month", "week", "year"]
    """
    Specifies billing duration. Either `day`, `week`, `month` or `year`.
    """
    interval_count: NotRequired[int]
    """
    The multiplier applied to the interval.
    """


class InvoiceCreatePreviewParamsSubscriptionDetails(TypedDict):
    billing_cycle_anchor: NotRequired["Literal['now', 'unchanged']|int"]
    """
    For new subscriptions, a future timestamp to anchor the subscription's [billing cycle](https://stripe.com/docs/subscriptions/billing-cycle). This is used to determine the date of the first full invoice, and, for plans with `month` or `year` intervals, the day of the month for subsequent invoices. For existing subscriptions, the value can only be set to `now` or `unchanged`.
    """
    billing_mode: NotRequired[
        "InvoiceCreatePreviewParamsSubscriptionDetailsBillingMode"
    ]
    """
    Controls how prorations and invoices for subscriptions are calculated and orchestrated.
    """
    billing_schedules: NotRequired[
        "Literal['']|List[InvoiceCreatePreviewParamsSubscriptionDetailsBillingSchedule]"
    ]
    """
    Sets the billing schedules for the subscription.
    """
    cancel_at: NotRequired[
        "Literal['']|int|Literal['max_period_end', 'min_period_end']"
    ]
    """
    A timestamp at which the subscription should cancel. If set to a date before the current period ends, this will cause a proration if prorations have been enabled using `proration_behavior`. If set during a future period, this will always cause a proration for that period.
    """
    cancel_at_period_end: NotRequired[bool]
    """
    Indicate whether this subscription should cancel at the end of the current period (`current_period_end`). Defaults to `false`.
    """
    cancel_now: NotRequired[bool]
    """
    This simulates the subscription being canceled or expired immediately.
    """
    default_tax_rates: NotRequired["Literal['']|List[str]"]
    """
    If provided, the invoice returned will preview updating or creating a subscription with these default tax rates. The default tax rates will apply to any line item that does not have `tax_rates` set.
    """
    items: NotRequired[
        List["InvoiceCreatePreviewParamsSubscriptionDetailsItem"]
    ]
    """
    A list of up to 20 subscription items, each with an attached price.
    """
    prebilling: NotRequired[
        "InvoiceCreatePreviewParamsSubscriptionDetailsPrebilling"
    ]
    """
    The pre-billing to apply to the subscription as a preview.
    """
    proration_behavior: NotRequired[
        Literal["always_invoice", "create_prorations", "none"]
    ]
    """
    Determines how to handle [prorations](https://stripe.com/docs/billing/subscriptions/prorations) when the billing cycle changes (e.g., when switching plans, resetting `billing_cycle_anchor=now`, or starting a trial), or if an item's `quantity` changes. The default value is `create_prorations`.
    """
    proration_date: NotRequired[int]
    """
    If previewing an update to a subscription, and doing proration, `subscription_details.proration_date` forces the proration to be calculated as though the update was done at the specified time. The time given must be within the current subscription period and within the current phase of the schedule backing this subscription, if the schedule exists. If set, `subscription`, and one of `subscription_details.items`, or `subscription_details.trial_end` are required. Also, `subscription_details.proration_behavior` cannot be set to 'none'.
    """
    resume_at: NotRequired[Literal["now"]]
    """
    For paused subscriptions, setting `subscription_details.resume_at` to `now` will preview the invoice that will be generated if the subscription is resumed.
    """
    start_date: NotRequired[int]
    """
    Date a subscription is intended to start (can be future or past).
    """
    trial_end: NotRequired["Literal['now']|int"]
    """
    If provided, the invoice returned will preview updating or creating a subscription with that trial end. If set, one of `subscription_details.items` or `subscription` is required.
    """


class InvoiceCreatePreviewParamsSubscriptionDetailsBillingMode(TypedDict):
    flexible: NotRequired[
        "InvoiceCreatePreviewParamsSubscriptionDetailsBillingModeFlexible"
    ]
    """
    Configure behavior for flexible billing mode.
    """
    type: Literal["classic", "flexible"]
    """
    Controls the calculation and orchestration of prorations and invoices for subscriptions. If no value is passed, the default is `flexible`.
    """


class InvoiceCreatePreviewParamsSubscriptionDetailsBillingModeFlexible(
    TypedDict,
):
    proration_discounts: NotRequired[Literal["included", "itemized"]]
    """
    Controls how invoices and invoice items display proration amounts and discount amounts.
    """


class InvoiceCreatePreviewParamsSubscriptionDetailsBillingSchedule(TypedDict):
    applies_to: NotRequired[
        List[
            "InvoiceCreatePreviewParamsSubscriptionDetailsBillingScheduleAppliesTo"
        ]
    ]
    """
    Configure billing schedule differently for individual subscription items.
    """
    bill_until: NotRequired[
        "InvoiceCreatePreviewParamsSubscriptionDetailsBillingScheduleBillUntil"
    ]
    """
    The end date for the billing schedule.
    """
    key: NotRequired[str]
    """
    Specify a key for the billing schedule. Must be unique to this field, alphanumeric, and up to 200 characters. If not provided, a unique key will be generated.
    """


class InvoiceCreatePreviewParamsSubscriptionDetailsBillingScheduleAppliesTo(
    TypedDict,
):
    price: NotRequired[str]
    """
    The ID of the price object.
    """
    type: Literal["price"]
    """
    Controls which subscription items the billing schedule applies to.
    """


class InvoiceCreatePreviewParamsSubscriptionDetailsBillingScheduleBillUntil(
    TypedDict,
):
    duration: NotRequired[
        "InvoiceCreatePreviewParamsSubscriptionDetailsBillingScheduleBillUntilDuration"
    ]
    """
    Specifies the billing period.
    """
    timestamp: NotRequired[int]
    """
    The end date of the billing schedule.
    """
    type: Literal[
        "amendment_end",
        "duration",
        "line_ends_at",
        "schedule_end",
        "timestamp",
        "upcoming_invoice",
    ]
    """
    Describes how the billing schedule will determine the end date. Either `duration` or `timestamp`.
    """


class InvoiceCreatePreviewParamsSubscriptionDetailsBillingScheduleBillUntilDuration(
    TypedDict,
):
    interval: Literal["day", "month", "week", "year"]
    """
    Specifies billing duration. Either `day`, `week`, `month` or `year`.
    """
    interval_count: NotRequired[int]
    """
    The multiplier applied to the interval.
    """


class InvoiceCreatePreviewParamsSubscriptionDetailsItem(TypedDict):
    billing_thresholds: NotRequired[
        "Literal['']|InvoiceCreatePreviewParamsSubscriptionDetailsItemBillingThresholds"
    ]
    """
    Define thresholds at which an invoice will be sent, and the subscription advanced to a new billing period. Pass an empty string to remove previously-defined thresholds.
    """
    clear_usage: NotRequired[bool]
    """
    Delete all usage for a given subscription item. You must pass this when deleting a usage records subscription item. `clear_usage` has no effect if the plan has a billing meter attached.
    """
    deleted: NotRequired[bool]
    """
    A flag that, if set to `true`, will delete the specified item.
    """
    discounts: NotRequired[
        "Literal['']|List[InvoiceCreatePreviewParamsSubscriptionDetailsItemDiscount]"
    ]
    """
    The coupons to redeem into discounts for the subscription item.
    """
    id: NotRequired[str]
    """
    Subscription item to update.
    """
    metadata: NotRequired["Literal['']|Dict[str, str]"]
    """
    Set of [key-value pairs](https://stripe.com/docs/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format. Individual keys can be unset by posting an empty value to them. All keys can be unset by posting an empty value to `metadata`.
    """
    plan: NotRequired[str]
    """
    Plan ID for this item, as a string.
    """
    price: NotRequired[str]
    """
    The ID of the price object. One of `price` or `price_data` is required. When changing a subscription item's price, `quantity` is set to 1 unless a `quantity` parameter is provided.
    """
    price_data: NotRequired[
        "InvoiceCreatePreviewParamsSubscriptionDetailsItemPriceData"
    ]
    """
    Data used to generate a new [Price](https://stripe.com/docs/api/prices) object inline. One of `price` or `price_data` is required.
    """
    quantity: NotRequired[int]
    """
    Quantity for this item.
    """
    tax_rates: NotRequired["Literal['']|List[str]"]
    """
    A list of [Tax Rate](https://stripe.com/docs/api/tax_rates) ids. These Tax Rates will override the [`default_tax_rates`](https://stripe.com/docs/api/subscriptions/create#create_subscription-default_tax_rates) on the Subscription. When updating, pass an empty string to remove previously-defined tax rates.
    """


class InvoiceCreatePreviewParamsSubscriptionDetailsItemBillingThresholds(
    TypedDict,
):
    usage_gte: int
    """
    Number of units that meets the billing threshold to advance the subscription to a new billing period (e.g., it takes 10 $5 units to meet a $50 [monetary threshold](https://stripe.com/docs/api/subscriptions/update#update_subscription-billing_thresholds-amount_gte))
    """


class InvoiceCreatePreviewParamsSubscriptionDetailsItemDiscount(TypedDict):
    coupon: NotRequired[str]
    """
    ID of the coupon to create a new discount for.
    """
    discount: NotRequired[str]
    """
    ID of an existing discount on the object (or one of its ancestors) to reuse.
    """
    discount_end: NotRequired[
        "InvoiceCreatePreviewParamsSubscriptionDetailsItemDiscountDiscountEnd"
    ]
    """
    Details to determine how long the discount should be applied for.
    """
    promotion_code: NotRequired[str]
    """
    ID of the promotion code to create a new discount for.
    """


class InvoiceCreatePreviewParamsSubscriptionDetailsItemDiscountDiscountEnd(
    TypedDict,
):
    duration: NotRequired[
        "InvoiceCreatePreviewParamsSubscriptionDetailsItemDiscountDiscountEndDuration"
    ]
    """
    Time span for the redeemed discount.
    """
    timestamp: NotRequired[int]
    """
    A precise Unix timestamp for the discount to end. Must be in the future.
    """
    type: Literal["duration", "timestamp"]
    """
    The type of calculation made to determine when the discount ends.
    """


class InvoiceCreatePreviewParamsSubscriptionDetailsItemDiscountDiscountEndDuration(
    TypedDict,
):
    interval: Literal["day", "month", "week", "year"]
    """
    Specifies a type of interval unit. Either `day`, `week`, `month` or `year`.
    """
    interval_count: int
    """
    The number of intervals, as an whole number greater than 0. Stripe multiplies this by the interval type to get the overall duration.
    """


class InvoiceCreatePreviewParamsSubscriptionDetailsItemPriceData(TypedDict):
    currency: str
    """
    Three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html), in lowercase. Must be a [supported currency](https://stripe.com/docs/currencies).
    """
    product: str
    """
    The ID of the [Product](https://docs.stripe.com/api/products) that this [Price](https://docs.stripe.com/api/prices) will belong to.
    """
    recurring: (
        "InvoiceCreatePreviewParamsSubscriptionDetailsItemPriceDataRecurring"
    )
    """
    The recurring components of a price such as `interval` and `interval_count`.
    """
    tax_behavior: NotRequired[Literal["exclusive", "inclusive", "unspecified"]]
    """
    Only required if a [default tax behavior](https://stripe.com/docs/tax/products-prices-tax-categories-tax-behavior#setting-a-default-tax-behavior-(recommended)) was not provided in the Stripe Tax settings. Specifies whether the price is considered inclusive of taxes or exclusive of taxes. One of `inclusive`, `exclusive`, or `unspecified`. Once specified as either `inclusive` or `exclusive`, it cannot be changed.
    """
    unit_amount: NotRequired[int]
    """
    A positive integer in cents (or local equivalent) (or 0 for a free price) representing how much to charge.
    """
    unit_amount_decimal: NotRequired[str]
    """
    Same as `unit_amount`, but accepts a decimal value in cents (or local equivalent) with at most 12 decimal places. Only one of `unit_amount` and `unit_amount_decimal` can be set.
    """


class InvoiceCreatePreviewParamsSubscriptionDetailsItemPriceDataRecurring(
    TypedDict,
):
    interval: Literal["day", "month", "week", "year"]
    """
    Specifies billing frequency. Either `day`, `week`, `month` or `year`.
    """
    interval_count: NotRequired[int]
    """
    The number of intervals between subscription billings. For example, `interval=month` and `interval_count=3` bills every 3 months. Maximum of three years interval allowed (3 years, 36 months, or 156 weeks).
    """


class InvoiceCreatePreviewParamsSubscriptionDetailsPrebilling(TypedDict):
    iterations: int
    """
    This is used to determine the number of billing cycles to prebill.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from typing import List
from typing_extensions import Literal, NotRequired, TypedDict


class ReportRunCreateParams(RequestOptions):
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    parameters: NotRequired["ReportRunCreateParamsParameters"]
    """
    Parameters specifying how the report should be run. Different Report Types have different required and optional parameters, listed in the [API Access to Reports](https://stripe.com/docs/reporting/statements/api) documentation.
    """
    report_type: str
    """
    The ID of the [report type](https://stripe.com/docs/reporting/statements/api#report-types) to run, such as `"balance.summary.1"`.
    """


class ReportRunCreateParamsParameters(TypedDict):
    columns: NotRequired[List[str]]
    """
    The set of report columns to include in the report output. If omitted, the Report Type is run with its default column set.
    """
    connected_account: NotRequired[str]
    """
    Connected account ID to filter for in the report run.
    """
    currency: NotRequired[str]
    """
    Currency of objects to be included in the report run.
    """
    interval_end: NotRequired[int]
    """
    Ending timestamp of data to be included in the report run (exclusive).
    """
    interval_start: NotRequired[int]
    """
    Starting timestamp of data to be included in the report run.
    """
    payout: NotRequired[str]
    """
    Payout ID by which to filter the report run.
    """
    reporting_category: NotRequired[
        Literal[
            "advance",
            "advance_funding",
            "anticipation_repayment",
            "charge",
            "charge_failure",
            "climate_order_purchase",
            "climate_order_refund",
            "connect_collection_transfer",
            "connect_reserved_funds",
            "contribution",
            "dispute",
            "dispute_reversal",
            "fee",
            "financing_paydown",
            "financing_paydown_reversal",
            "financing_payout",
            "financing_payout_reversal",
            "issuing_authorization_hold",
            "issuing_authorization_release",
            "issuing_dispute",
            "issuing_transaction",
            "network_cost",
            "other_adjustment",
            "partial_capture_reversal",
            "payout",
            "payout_reversal",
            "platform_earning",
            "platform_earning_refund",
            "refund",
            "refund_failure",
            "risk_reserved_funds",
            "tax",
            "topup",
            "topup_reversal",
            "transfer",
            "transfer_reversal",
            "unreconciled_customer_funds",
        ]
    ]
    """
    Category of balance transactions to be included in the report run.
    """
    timezone: NotRequired[
        Literal[
            "Africa/Abidjan",
            "Africa/Accra",
            "Africa/Addis_Ababa",
            "Africa/Algiers",
            "Africa/Asmara",
            "Africa/Asmera",
            "Africa/Bamako",
            "Africa/Bangui",
            "Africa/Banjul",
            "Africa/Bissau",
            "Africa/Blantyre",
            "Africa/Brazzaville",
            "Africa/Bujumbura",
            "Africa/Cairo",
            "Africa/Casablanca",
            "Africa/Ceuta",
            "Africa/Conakry",
            "Africa/Dakar",
            "Africa/Dar_es_Salaam",
            "Africa/Djibouti",
            "Africa/Douala",
            "Africa/El_Aaiun",
            "Africa/Freetown",
            "Africa/Gaborone",
            "Africa/Harare",
            "Africa/Johannesburg",
            "Africa/Juba",
            "Africa/Kampala",
            "Africa/Khartoum",
            "Africa/Kigali",
            "Africa/Kinshasa",
            "Africa/Lagos",
            "Africa/Libreville",
            "Africa/Lome",
            "Africa/Luanda",
            "Africa/Lubumbashi",
            "Africa/Lusaka",
            "Africa/Malabo",
            "Africa/Maputo",
            "Africa/Maseru",
            "Africa/Mbabane",
            "Africa/Mogadishu",
            "Africa/Monrovia",
            "Africa/Nairobi",
            "Africa/Ndjamena",
            "Africa/Niamey",
            "Africa/Nouakchott",
            "Africa/Ouagadougou",
            "Africa/Porto-Novo",
            "Africa/Sao_Tome",
            "Africa/Timbuktu",
            "Africa/Tripoli",
            "Africa/Tunis",
            "Africa/Windhoek",
            "America/Adak",
            "America/Anchorage",
            "America/Anguilla",
            "America/Antigua",
            "America/Araguaina",
            "America/Argentina/Buenos_Aires",
            "America/Argentina/Catamarca",
            "America/Argentina/ComodRivadavia",
            "America/Argentina/Cordoba",
            "America/Argentina/Jujuy",
            "America/Argentina/La_Rioja",
            "America/Argentina/Mendoza",
            "America/Argentina/Rio_Gallegos",
            "America/Argentina/Salta",
            "America/Argentina/San_Juan",
            "America/Argentina/San_Luis",
            "America/Argentina/Tucuman",
            "America/Argentina/Ushuaia",
            "America/Aruba",
            "America/Asuncion",
            "America/Atikokan",
            "America/Atka",
            "America/Bahia",
            "America/Bahia_Banderas",
            "America/Barbados",
            "America/Belem",
            "America/Belize",
            "America/Blanc-Sablon",
            "America/Boa_Vista",
            "America/Bogota",
            "America/Boise",
            "America/Buenos_Aires",
            "America/Cambridge_Bay",
            "America/Campo_Grande",
            "America/Cancun",
            "America/Caracas",
            "America/Catamarca",
            "America/Cayenne",
            "America/Cayman",
            "America/Chicago",
            "America/Chihuahua",
            "America/Ciudad_Juarez",
            "America/Coral_Harbour",
            "America/Cordoba",
            "America/Costa_Rica",
            "America/Coyhaique",
            "America/Creston",
            "America/Cuiaba",
            "America/Curacao",
            "America/Danmarkshavn",
            "America/Dawson",
            "America/Dawson_Creek",
            "America/Denver",
            "America/Detroit",
            "America/Dominica",
            "America/Edmonton",
            "America/Eirunepe",
            "America/El_Salvador",
            "America/Ensenada",
            "America/Fort_Nelson",
            "America/Fort_Wayne",
            "America/Fortaleza",
            "America/Glace_Bay",
            "America/Godthab",
            "America/Goose_Bay",
            "America/Grand_Turk",
            "America/Grenada",
            "America/Guadeloupe",
            "America/Guatemala",
            "America/Guayaquil",
            "America/Guyana",
            "America/Halifax",
            "America/Havana",
            "America/Hermosillo",
            "America/Indiana/Indianapolis",
            "America/Indiana/Knox",
            "America/Indiana/Marengo",
            "America/Indiana/Petersburg",
            "America/Indiana/Tell_City",
            "America/Indiana/Vevay",
            "America/Indiana/Vincennes",
            "America/Indiana/Winamac",
            "America/Indianapolis",
            "America/Inuvik",
            "America/Iqaluit",
            "America/Jamaica",
            "America/Jujuy",
            "America/Juneau",
            "America/Kentucky/Louisville",
            "America/Kentucky/Monticello",
            "America/Knox_IN",
            "America/Kralendijk",
            "America/La_Paz",
            "America/Lima",
            "America/Los_Angeles",
            "America/Louisville",
            "America/Lower_Princes",
            "America/Maceio",
            "America/Managua",
            "America/Manaus",
            "America/Marigot",
            "America/Martinique",
            "America/Matamoros",
            "America/Mazatlan",
            "America/Mendoza",
            "America/Menominee",
            "America/Merida",
            "America/Metlakatla",
            "America/Mexico_City",
            "America/Miquelon",
            "America/Moncton",
            "America/Monterrey",
            "America/Montevideo",
            "America/Montreal",
            "America/Montserrat",
            "America/Nassau",
            "America/New_York",
            "America/Nipigon",
            "America/Nome",
            "America/Noronha",
            "America/North_Dakota/Beulah",
            "America/North_Dakota/Center",
            "America/North_Dakota/New_Salem",
            "America/Nuuk",
            "America/Ojinaga",
            "America/Panama",
            "America/Pangnirtung",
            "America/Paramaribo",
            "America/Phoenix",
            "America/Port-au-Prince",
            "America/Port_of_Spain",
            "America/Porto_Acre",
            "America/Porto_Velho",
            "America/Puerto_Rico",
            "America/Punta_Arenas",
            "America/Rainy_River",
            "America/Rankin_Inlet",
            "America/Recife",
            "America/Regina",
            "America/Resolute",
            "America/Rio_Branco",
            "America/Rosario",
            "America/Santa_Isabel",
            "America/Santarem",
            "America/Santiago",
            "America/Santo_Domingo",
            "America/Sao_Paulo",
            "America/Scoresbysund",
            "America/Shiprock",
            "America/Sitka",
            "America/St_Barthelemy",
            "America/St_Johns",
            "America/St_Kitts",
            "America/St_Lucia",
            "America/St_Thomas",
            "America/St_Vincent",
            "America/Swift_Current",
            "America/Tegucigalpa",
            "America/Thule",
            "America/Thunder_Bay",
            "America/Tijuana",
            "America/Toronto",
            "America/Tortola",
            "America/Vancouver",
            "America/Virgin",
            "America/Whitehorse",
            "America/Winnipeg",
            "America/Yakutat",
            "America/Yellowknife",
            "Antarctica/Casey",
            "Antarctica/Davis",
            "Antarctica/DumontDUrville",
            "Antarctica/Macquarie",
            "Antarctica/Mawson",
            "Antarctica/McMurdo",
            "Antarctica/Palmer",
            "Antarctica/Rothera",
            "Antarctica/South_Pole",
            "Antarctica/Syowa",
            "Antarctica/Troll",
            "Antarctica/Vostok",
            "Arctic/Longyearbyen",
            "Asia/Aden",
            "Asia/Almaty",
            "Asia/Amman",
            "Asia/Anadyr",
            "Asia/Aqtau",
            "Asia/Aqtobe",
            "Asia/Ashgabat",
            "Asia/Ashkhabad",
            "Asia/Atyrau",
            "Asia/Baghdad",
            "Asia/Bahrain",
            "Asia/Baku",
            "Asia/Bangkok",
            "Asia/Barnaul",
            "Asia/Beirut",
            "Asia/Bishkek",
            "Asia/Brunei",
            "Asia/Calcutta",
            "Asia/Chita",
            "Asia/Choibalsan",
            "Asia/Chongqing",
            "Asia/Chungking",
            "Asia/Colombo",
            "Asia/Dacca",
            "Asia/Damascus",
            "Asia/Dhaka",
            "Asia/Dili",
            "Asia/Dubai",
            "Asia/Dushanbe",
            "Asia/Famagusta",
            "Asia/Gaza",
            "Asia/Harbin",
            "Asia/Hebron",
            "Asia/Ho_Chi_Minh",
            "Asia/Hong_Kong",
            "Asia/Hovd",
            "Asia/Irkutsk",
            "Asia/Istanbul",
            "Asia/Jakarta",
            "Asia/Jayapura",
            "Asia/Jerusalem",
            "Asia/Kabul",
            "Asia/Kamchatka",
            "Asia/Karachi",
            "Asia/Kashgar",
            "Asia/Kathmandu",
            "Asia/Katmandu",
            "Asia/Khandyga",
            "Asia/Kolkata",
            "Asia/Krasnoyarsk",
            "Asia/Kuala_Lumpur",
            "Asia/Kuching",
            "Asia/Kuwait",
            "Asia/Macao",
            "Asia/Macau",
            "Asia/Magadan",
            "Asia/Makassar",
            "Asia/Manila",
            "Asia/Muscat",
            "Asia/Nicosia",
            "Asia/Novokuznetsk",
            "Asia/Novosibirsk",
            "Asia/Omsk",
            "Asia/Oral",
            "Asia/Phnom_Penh",
            "Asia/Pontianak",
            "Asia/Pyongyang",
            "Asia/Qatar",
            "Asia/Qostanay",
            "Asia/Qyzylorda",
            "Asia/Rangoon",
            "Asia/Riyadh",
            "Asia/Saigon",
            "Asia/Sakhalin",
            "Asia/Samarkand",
            "Asia/Seoul",
            "Asia/Shanghai",
            "Asia/Singapore",
            "Asia/Srednekolymsk",
            "Asia/Taipei",
            "Asia/Tashkent",
            "Asia/Tbilisi",
            "Asia/Tehran",
            "Asia/Tel_Aviv",
            "Asia/Thimbu",
            "Asia/Thimphu",
            "Asia/Tokyo",
            "Asia/Tomsk",
            "Asia/Ujung_Pandang",
            "Asia/Ulaanbaatar",
            "Asia/Ulan_Bator",
            "Asia/Urumqi",
            "Asia/Ust-Nera",
            "Asia/Vientiane",
            "Asia/Vladivostok",
            "Asia/Yakutsk",
            "Asia/Yangon",
            "Asia/Yekaterinburg",
            "Asia/Yerevan",
            "Atlantic/Azores",
            "Atlantic/Bermuda",
            "Atlantic/Canary",
            "Atlantic/Cape_Verde",
            "Atlantic/Faeroe",
            "Atlantic/Faroe",
            "Atlantic/Jan_Mayen",
            "Atlantic/Madeira",
            "Atlantic/Reykjavik",
            "Atlantic/South_Georgia",
            "Atlantic/St_Helena",
            "Atlantic/Stanley",
            "Australia/ACT",
            "Australia/Adelaide",
            "Australia/Brisbane",
            "Australia/Broken_Hill",
            "Australia/Canberra",
            "Australia/Currie",
            "Australia/Darwin",
            "Australia/Eucla",
            "Australia/Hobart",
            "Australia/LHI",
            "Australia/Lindeman",
            "Australia/Lord_Howe",
            "Australia/Melbourne",
            "Australia/NSW",
            "Australia/North",
            "Australia/Perth",
            "Australia/Queensland",
            "Australia/South",
            "Australia/Sydney",
            "Australia/Tasmania",
            "Australia/Victoria",
            "Australia/West",
            "Australia/Yancowinna",
            "Brazil/Acre",
            "Brazil/DeNoronha",
            "Brazil/East",
            "Brazil/West",
            "CET",
            "CST6CDT",
            "Canada/Atlantic",
            "Canada/Central",
            "Canada/Eastern",
            "Canada/Mountain",
            "Canada/Newfoundland",
            "Canada/Pacific",
            "Canada/Saskatchewan",
            "Canada/Yukon",
            "Chile/Continental",
            "Chile/EasterIsland",
            "Cuba",
            "EET",
            "EST",
            "EST5EDT",
            "Egypt",
            "Eire",
            "Etc/GMT",
            "Etc/GMT+0",
            "Etc/GMT+1",
            "Etc/GMT+10",
            "Etc/GMT+11",
            "Etc/GMT+12",
            "Etc/GMT+2",
            "Etc/GMT+3",
            "Etc/GMT+4",
            "Etc/GMT+5",
            "Etc/GMT+6",
            "Etc/GMT+7",
            "Etc/GMT+8",
            "Etc/GMT+9",
            "Etc/GMT-0",
            "Etc/GMT-1",
            "Etc/GMT-10",
            "Etc/GMT-11",
            "Etc/GMT-12",
            "Etc/GMT-13",
            "Etc/GMT-14",
            "Etc/GMT-2",
            "Etc/GMT-3",
            "Etc/GMT-4",
            "Etc/GMT-5",
            "Etc/GMT-6",
            "Etc/GMT-7",
            "Etc/GMT-8",
            "Etc/GMT-9",
            "Etc/GMT0",
            "Etc/Greenwich",
            "Etc/UCT",
            "Etc/UTC",
            "Etc/Universal",
            "Etc/Zulu",
            "Europe/Amsterdam",
            "Europe/Andorra",
            "Europe/Astrakhan",
            "Europe/Athens",
            "Europe/Belfast",
            "Europe/Belgrade",
            "Europe/Berlin",
            "Europe/Bratislava",
            "Europe/Brussels",
            "Europe/Bucharest",
            "Europe/Budapest",
            "Europe/Busingen",
            "Europe/Chisinau",
            "Europe/Copenhagen",
            "Europe/Dublin",
            "Europe/Gibraltar",
            "Europe/Guernsey",
            "Europe/Helsinki",
            "Europe/Isle_of_Man",
            "Europe/Istanbul",
            "Europe/Jersey",
            "Europe/Kaliningrad",
            "Europe/Kiev",
            "Europe/Kirov",
            "Europe/Kyiv",
            "Europe/Lisbon",
            "Europe/Ljubljana",
            "Europe/London",
            "Europe/Luxembourg",
            "Europe/Madrid",
            "Europe/Malta",
            "Europe/Mariehamn",
            "Europe/Minsk",
            "Europe/Monaco",
            "Europe/Moscow",
            "Europe/Nicosia",
            "Europe/Oslo",
            "Europe/Paris",
            "Europe/Podgorica",
            "Europe/Prague",
            "Europe/Riga",
            "Europe/Rome",
            "Europe/Samara",
            "Europe/San_Marino",
            "Europe/Sarajevo",
            "Europe/Saratov",
            "Europe/Simferopol",
            "Europe/Skopje",
            "Europe/Sofia",
            "Europe/Stockholm",
            "Europe/Tallinn",
            "Europe/Tirane",
            "Europe/Tiraspol",
            "Europe/Ulyanovsk",
            "Europe/Uzhgorod",
            "Europe/Vaduz",
            "Europe/Vatican",
            "Europe/Vienna",
            "Europe/Vilnius",
            "Europe/Volgograd",
            "Europe/Warsaw",
            "Europe/Zagreb",
            "Europe/Zaporozhye",
            "Europe/Zurich",
            "Factory",
            "GB",
            "GB-Eire",
            "GMT",
            "GMT+0",
            "GMT-0",
            "GMT0",
            "Greenwich",
            "HST",
            "Hongkong",
            "Iceland",
            "Indian/Antananarivo",
            "Indian/Chagos",
            "Indian/Christmas",
            "Indian/Cocos",
            "Indian/Comoro",
            "Indian/Kerguelen",
            "Indian/Mahe",
            "Indian/Maldives",
            "Indian/Mauritius",
            "Indian/Mayotte",
            "Indian/Reunion",
            "Iran",
            "Israel",
            "Jamaica",
            "Japan",
            "Kwajalein",
            "Libya",
            "MET",
            "MST",
            "MST7MDT",
            "Mexico/BajaNorte",
            "Mexico/BajaSur",
            "Mexico/General",
            "NZ",
            "NZ-CHAT",
            "Navajo",
            "PRC",
            "PST8PDT",
            "Pacific/Apia",
            "Pacific/Auckland",
            "Pacific/Bougainville",
            "Pacific/Chatham",
            "Pacific/Chuuk",
            "Pacific/Easter",
            "Pacific/Efate",
            "Pacific/Enderbury",
            "Pacific/Fakaofo",
            "Pacific/Fiji",
            "Pacific/Funafuti",
            "Pacific/Galapagos",
            "Pacific/Gambier",
            "Pacific/Guadalcanal",
            "Pacific/Guam",
            "Pacific/Honolulu",
            "Pacific/Johnston",
            "Pacific/Kanton",
            "Pacific/Kiritimati",
            "Pacific/Kosrae",
            "Pacific/Kwajalein",
            "Pacific/Majuro",
            "Pacific/Marquesas",
            "Pacific/Midway",
            "Pacific/Nauru",
            "Pacific/Niue",
            "Pacific/Norfolk",
            "Pacific/Noumea",
            "Pacific/Pago_Pago",
            "Pacific/Palau",
            "Pacific/Pitcairn",
            "Pacific/Pohnpei",
            "Pacific/Ponape",
            "Pacific/Port_Moresby",
            "Pacific/Rarotonga",
            "Pacific/Saipan",
            "Pacific/Samoa",
            "Pacific/Tahiti",
            "Pacific/Tarawa",
            "Pacific/Tongatapu",
            "Pacific/Truk",
            "Pacific/Wake",
            "Pacific/Wallis",
            "Pacific/Yap",
            "Poland",
            "Portugal",
            "ROC",
            "ROK",
            "Singapore",
            "Turkey",
            "UCT",
            "US/Alaska",
            "US/Aleutian",
            "US/Arizona",
            "US/Central",
            "US/East-Indiana",
            "US/Eastern",
            "US/Hawaii",
            "US/Indiana-Starke",
            "US/Michigan",
            "US/Mountain",
            "US/Pacific",
            "US/Pacific-New",
            "US/Samoa",
            "UTC",
            "Universal",
            "W-SU",
            "WET",
            "Zulu",
        ]
    ]
    """
    Defaults to `Etc/UTC`. The output timezone for all timestamps in the report. A list of possible time zone values is maintained at the [IANA Time Zone Database](http://www.iana.org/time-zones). Has no effect on `interval_start` or `interval_end`.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_object import StripeObject
from typing import ClassVar, List, Optional
from typing_extensions import Literal


class AccountLink(StripeObject):
    """
    AccountLinks are the means by which a Merchant grants an Account permission to access Stripe-hosted applications, such as Recipient Onboarding. This API is only available for users enrolled in the public preview for Accounts v2.
    """

    OBJECT_NAME: ClassVar[Literal["v2.core.account_link"]] = (
        "v2.core.account_link"
    )

    class UseCase(StripeObject):
        class AccountOnboarding(StripeObject):
            class CollectionOptions(StripeObject):
                fields: Optional[Literal["currently_due", "eventually_due"]]
                """
                Specifies whether the platform collects only currently_due requirements (`currently_due`) or both currently_due and eventually_due requirements (`eventually_due`). If you don't specify collection_options, the default value is currently_due.
                """
                future_requirements: Optional[Literal["include", "omit"]]
                """
                Specifies whether the platform collects future_requirements in addition to requirements in Connect Onboarding. The default value is `omit`.
                """

            collection_options: Optional[CollectionOptions]
            """
            Specifies the requirements that Stripe collects from v2/core/accounts in the Onboarding flow.
            """
            configurations: List[
                Literal["customer", "merchant", "recipient", "storer"]
            ]
            """
            Open Enum. A v2/core/account can be configured to enable certain functionality. The configuration param targets the v2/core/account_link to collect information for the specified v2/core/account configuration/s.
            """
            refresh_url: str
            """
            The URL the user will be redirected to if the AccountLink is expired, has been used, or is otherwise invalid. The URL you specify should attempt to generate a new AccountLink with the same parameters used to create the original AccountLink, then redirect the user to the new AccountLink's URL so they can continue the flow. If a new AccountLink cannot be generated or the redirect fails you should display a useful error to the user. Please make sure to implement authentication before redirecting the user in case this URL is leaked to a third party.
            """
            return_url: Optional[str]
            """
            The URL that the user will be redirected to upon completing the linked flow.
            """
            _inner_class_types = {"collection_options": CollectionOptions}

        class AccountUpdate(StripeObject):
            class CollectionOptions(StripeObject):
                fields: Optional[Literal["currently_due", "eventually_due"]]
                """
                Specifies whether the platform collects only currently_due requirements (`currently_due`) or both currently_due and eventually_due requirements (`eventually_due`). If you don't specify collection_options, the default value is currently_due.
                """
                future_requirements: Optional[Literal["include", "omit"]]
                """
                Specifies whether the platform collects future_requirements in addition to requirements in Connect Onboarding. The default value is `omit`.
                """

            collection_options: Optional[CollectionOptions]
            """
            Specifies the requirements that Stripe collects from v2/core/accounts in the Onboarding flow.
            """
            configurations: List[
                Literal["customer", "merchant", "recipient", "storer"]
            ]
            """
            Open Enum. A v2/account can be configured to enable certain functionality. The configuration param targets the v2/account_link to collect information for the specified v2/account configuration/s.
            """
            refresh_url: str
            """
            The URL the user will be redirected to if the AccountLink is expired, has been used, or is otherwise invalid. The URL you specify should attempt to generate a new AccountLink with the same parameters used to create the original AccountLink, then redirect the user to the new AccountLink's URL so they can continue the flow. If a new AccountLink cannot be generated or the redirect fails you should display a useful error to the user. Please make sure to implement authentication before redirecting the user in case this URL is leaked to a third party.
            """
            return_url: Optional[str]
            """
            The URL that the user will be redirected to upon completing the linked flow.
            """
            _inner_class_types = {"collection_options": CollectionOptions}

        account_onboarding: Optional[AccountOnboarding]
        """
        Indicates that the AccountLink provided should onboard an account.
        """
        account_update: Optional[AccountUpdate]
        """
        Indicates that the AccountLink provided should update a previously onboarded account.
        """
        type: Literal["account_onboarding", "account_update"]
        """
        Open Enum. The type of AccountLink the user is requesting.
        """
        _inner_class_types = {
            "account_onboarding": AccountOnboarding,
            "account_update": AccountUpdate,
        }

    account: str
    """
    The ID of the Account the link was created for.
    """
    created: str
    """
    The timestamp at which this AccountLink was created.
    """
    expires_at: str
    """
    The timestamp at which this AccountLink will expire.
    """
    livemode: bool
    """
    Has the value `true` if the object exists in live mode or the value `false` if the object exists in test mode.
    """
    object: Literal["v2.core.account_link"]
    """
    String representing the object's type. Objects of the same type share the same value of the object field.
    """
    url: str
    """
    The URL for the AccountLink.
    """
    use_case: UseCase
    """
    The use case of AccountLink the user is requesting.
    """
    _inner_class_types = {"use_case": UseCase}



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from importlib import import_module
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe.tax._association_service import AssociationService
    from stripe.tax._calculation_service import CalculationService
    from stripe.tax._form_service import FormService
    from stripe.tax._registration_service import RegistrationService
    from stripe.tax._settings_service import SettingsService
    from stripe.tax._transaction_service import TransactionService

_subservices = {
    "associations": ["stripe.tax._association_service", "AssociationService"],
    "calculations": ["stripe.tax._calculation_service", "CalculationService"],
    "forms": ["stripe.tax._form_service", "FormService"],
    "registrations": [
        "stripe.tax._registration_service",
        "RegistrationService",
    ],
    "settings": ["stripe.tax._settings_service", "SettingsService"],
    "transactions": ["stripe.tax._transaction_service", "TransactionService"],
}


class TaxService(StripeService):
    associations: "AssociationService"
    calculations: "CalculationService"
    forms: "FormService"
    registrations: "RegistrationService"
    settings: "SettingsService"
    transactions: "TransactionService"

    def __init__(self, requestor):
        super().__init__(requestor)

    def __getattr__(self, name):
        try:
            import_from, service = _subservices[name]
            service_class = getattr(
                import_module(import_from),
                service,
            )
            setattr(
                self,
                name,
                service_class(self._requestor),
            )
            return getattr(self, name)
        except KeyError:
            raise AttributeError()



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._request_options import RequestOptions
    from stripe.params.v2.core.vault._us_bank_account_archive_params import (
        UsBankAccountArchiveParams,
    )
    from stripe.params.v2.core.vault._us_bank_account_confirm_microdeposits_params import (
        UsBankAccountConfirmMicrodepositsParams,
    )
    from stripe.params.v2.core.vault._us_bank_account_create_params import (
        UsBankAccountCreateParams,
    )
    from stripe.params.v2.core.vault._us_bank_account_list_params import (
        UsBankAccountListParams,
    )
    from stripe.params.v2.core.vault._us_bank_account_retrieve_params import (
        UsBankAccountRetrieveParams,
    )
    from stripe.params.v2.core.vault._us_bank_account_send_microdeposits_params import (
        UsBankAccountSendMicrodepositsParams,
    )
    from stripe.params.v2.core.vault._us_bank_account_update_params import (
        UsBankAccountUpdateParams,
    )
    from stripe.v2._list_object import ListObject
    from stripe.v2.core.vault._us_bank_account import UsBankAccount


class UsBankAccountService(StripeService):
    def list(
        self,
        params: Optional["UsBankAccountListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[UsBankAccount]":
        """
        List USBankAccount objects. Optionally filter by verification status.
        """
        return cast(
            "ListObject[UsBankAccount]",
            self._request(
                "get",
                "/v2/core/vault/us_bank_accounts",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        params: Optional["UsBankAccountListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[UsBankAccount]":
        """
        List USBankAccount objects. Optionally filter by verification status.
        """
        return cast(
            "ListObject[UsBankAccount]",
            await self._request_async(
                "get",
                "/v2/core/vault/us_bank_accounts",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def create(
        self,
        params: "UsBankAccountCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "UsBankAccount":
        """
        Create a USBankAccount object.
        """
        return cast(
            "UsBankAccount",
            self._request(
                "post",
                "/v2/core/vault/us_bank_accounts",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self,
        params: "UsBankAccountCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "UsBankAccount":
        """
        Create a USBankAccount object.
        """
        return cast(
            "UsBankAccount",
            await self._request_async(
                "post",
                "/v2/core/vault/us_bank_accounts",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        id: str,
        params: Optional["UsBankAccountRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "UsBankAccount":
        """
        Retrieve a USBankAccount object.
        """
        return cast(
            "UsBankAccount",
            self._request(
                "get",
                "/v2/core/vault/us_bank_accounts/{id}".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        id: str,
        params: Optional["UsBankAccountRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "UsBankAccount":
        """
        Retrieve a USBankAccount object.
        """
        return cast(
            "UsBankAccount",
            await self._request_async(
                "get",
                "/v2/core/vault/us_bank_accounts/{id}".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def update(
        self,
        id: str,
        params: Optional["UsBankAccountUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "UsBankAccount":
        """
        Update a USBankAccount object. This is limited to supplying a previously empty routing_number field.
        """
        return cast(
            "UsBankAccount",
            self._request(
                "post",
                "/v2/core/vault/us_bank_accounts/{id}".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def update_async(
        self,
        id: str,
        params: Optional["UsBankAccountUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "UsBankAccount":
        """
        Update a USBankAccount object. This is limited to supplying a previously empty routing_number field.
        """
        return cast(
            "UsBankAccount",
            await self._request_async(
                "post",
                "/v2/core/vault/us_bank_accounts/{id}".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def archive(
        self,
        id: str,
        params: Optional["UsBankAccountArchiveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "UsBankAccount":
        """
        Archive a USBankAccount object. USBankAccount objects will not be automatically archived by Stripe.
        Archived USBankAccount objects cannot be used as outbound destinations
        and will not appear in the outbound destination list.
        """
        return cast(
            "UsBankAccount",
            self._request(
                "post",
                "/v2/core/vault/us_bank_accounts/{id}/archive".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def archive_async(
        self,
        id: str,
        params: Optional["UsBankAccountArchiveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "UsBankAccount":
        """
        Archive a USBankAccount object. USBankAccount objects will not be automatically archived by Stripe.
        Archived USBankAccount objects cannot be used as outbound destinations
        and will not appear in the outbound destination list.
        """
        return cast(
            "UsBankAccount",
            await self._request_async(
                "post",
                "/v2/core/vault/us_bank_accounts/{id}/archive".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def confirm_microdeposits(
        self,
        id: str,
        params: Optional["UsBankAccountConfirmMicrodepositsParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "UsBankAccount":
        """
        Confirm microdeposits amounts or descriptor code that you have received from the Send Microdeposits request. Once you correctly confirm this, this US Bank Account will be verified and eligible to transfer funds with.
        """
        return cast(
            "UsBankAccount",
            self._request(
                "post",
                "/v2/core/vault/us_bank_accounts/{id}/confirm_microdeposits".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def confirm_microdeposits_async(
        self,
        id: str,
        params: Optional["UsBankAccountConfirmMicrodepositsParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "UsBankAccount":
        """
        Confirm microdeposits amounts or descriptor code that you have received from the Send Microdeposits request. Once you correctly confirm this, this US Bank Account will be verified and eligible to transfer funds with.
        """
        return cast(
            "UsBankAccount",
            await self._request_async(
                "post",
                "/v2/core/vault/us_bank_accounts/{id}/confirm_microdeposits".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def send_microdeposits(
        self,
        id: str,
        params: Optional["UsBankAccountSendMicrodepositsParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "UsBankAccount":
        """
        Send microdeposits in order to verify your US Bank Account so it is eligible to transfer funds. This will start the verification process and you must Confirm Microdeposits to successfully verify your US Bank Account.
        """
        return cast(
            "UsBankAccount",
            self._request(
                "post",
                "/v2/core/vault/us_bank_accounts/{id}/send_microdeposits".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def send_microdeposits_async(
        self,
        id: str,
        params: Optional["UsBankAccountSendMicrodepositsParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "UsBankAccount":
        """
        Send microdeposits in order to verify your US Bank Account so it is eligible to transfer funds. This will start the verification process and you must Confirm Microdeposits to successfully verify your US Bank Account.
        """
        return cast(
            "UsBankAccount",
            await self._request_async(
                "post",
                "/v2/core/vault/us_bank_accounts/{id}/send_microdeposits".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from typing import List
from typing_extensions import NotRequired, TypedDict


class BalanceTransactionListParams(RequestOptions):
    created: NotRequired["BalanceTransactionListParamsCreated|int"]
    """
    Only return transactions that were created during the given date interval.
    """
    currency: NotRequired[str]
    """
    Only return transactions in a certain currency. Three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html), in lowercase. Must be a [supported currency](https://stripe.com/docs/currencies).
    """
    ending_before: NotRequired[str]
    """
    A cursor for use in pagination. `ending_before` is an object ID that defines your place in the list. For instance, if you make a list request and receive 100 objects, starting with `obj_bar`, your subsequent call can include `ending_before=obj_bar` in order to fetch the previous page of the list.
    """
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    limit: NotRequired[int]
    """
    A limit on the number of objects to be returned. Limit can range between 1 and 100, and the default is 10.
    """
    payout: NotRequired[str]
    """
    For automatic Stripe payouts only, only returns transactions that were paid out on the specified payout ID.
    """
    source: NotRequired[str]
    """
    Only returns the original transaction.
    """
    starting_after: NotRequired[str]
    """
    A cursor for use in pagination. `starting_after` is an object ID that defines your place in the list. For instance, if you make a list request and receive 100 objects, ending with `obj_foo`, your subsequent call can include `starting_after=obj_foo` in order to fetch the next page of the list.
    """
    type: NotRequired[str]
    """
    Only returns transactions of the given type. One of: `adjustment`, `advance`, `advance_funding`, `anticipation_repayment`, `application_fee`, `application_fee_refund`, `charge`, `climate_order_purchase`, `climate_order_refund`, `connect_collection_transfer`, `contribution`, `issuing_authorization_hold`, `issuing_authorization_release`, `issuing_dispute`, `issuing_transaction`, `obligation_outbound`, `obligation_reversal_inbound`, `payment`, `payment_failure_refund`, `payment_network_reserve_hold`, `payment_network_reserve_release`, `payment_refund`, `payment_reversal`, `payment_unreconciled`, `payout`, `payout_cancel`, `payout_failure`, `payout_minimum_balance_hold`, `payout_minimum_balance_release`, `refund`, `refund_failure`, `reserve_transaction`, `reserved_funds`, `stripe_fee`, `stripe_fx_fee`, `stripe_balance_payment_debit`, `stripe_balance_payment_debit_reversal`, `tax_fee`, `topup`, `topup_reversal`, `transfer`, `transfer_cancel`, `transfer_failure`, or `transfer_refund`.
    """


class BalanceTransactionListParamsCreated(TypedDict):
    gt: NotRequired[int]
    """
    Minimum value to filter by (exclusive)
    """
    gte: NotRequired[int]
    """
    Minimum value to filter by (inclusive)
    """
    lt: NotRequired[int]
    """
    Maximum value to filter by (exclusive)
    """
    lte: NotRequired[int]
    """
    Maximum value to filter by (inclusive)
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._ephemeral_key import EphemeralKey
    from stripe._request_options import RequestOptions
    from stripe.params._ephemeral_key_create_params import (
        EphemeralKeyCreateParams,
    )
    from stripe.params._ephemeral_key_delete_params import (
        EphemeralKeyDeleteParams,
    )


class EphemeralKeyService(StripeService):
    def delete(
        self,
        key: str,
        params: Optional["EphemeralKeyDeleteParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "EphemeralKey":
        """
        Invalidates a short-lived API key for a given resource.
        """
        return cast(
            "EphemeralKey",
            self._request(
                "delete",
                "/v1/ephemeral_keys/{key}".format(key=sanitize_id(key)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def delete_async(
        self,
        key: str,
        params: Optional["EphemeralKeyDeleteParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "EphemeralKey":
        """
        Invalidates a short-lived API key for a given resource.
        """
        return cast(
            "EphemeralKey",
            await self._request_async(
                "delete",
                "/v1/ephemeral_keys/{key}".format(key=sanitize_id(key)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def create(
        self,
        params: Optional["EphemeralKeyCreateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "EphemeralKey":
        """
        Creates a short-lived API key for a given resource.
        """
        return cast(
            "EphemeralKey",
            self._request(
                "post",
                "/v1/ephemeral_keys",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self,
        params: Optional["EphemeralKeyCreateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "EphemeralKey":
        """
        Creates a short-lived API key for a given resource.
        """
        return cast(
            "EphemeralKey",
            await self._request_async(
                "post",
                "/v1/ephemeral_keys",
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from typing import List
from typing_extensions import NotRequired, TypedDict


class RequestListParams(RequestOptions):
    created: NotRequired["RequestListParamsCreated"]
    """
    Similar to other List endpoints, filters results based on created timestamp. You can pass gt, gte, lt, and lte timestamp values.
    """
    ending_before: NotRequired[str]
    """
    A pagination cursor to fetch the previous page of the list. The value must be a ForwardingRequest ID.
    """
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    limit: NotRequired[int]
    """
    A limit on the number of objects to be returned. Limit can range between 1 and 100, and the default is 10.
    """
    starting_after: NotRequired[str]
    """
    A pagination cursor to fetch the next page of the list. The value must be a ForwardingRequest ID.
    """


class RequestListParamsCreated(TypedDict):
    gt: NotRequired[int]
    """
    Return results where the `created` field is greater than this value.
    """
    gte: NotRequired[int]
    """
    Return results where the `created` field is greater than or equal to this value.
    """
    lt: NotRequired[int]
    """
    Return results where the `created` field is less than this value.
    """
    lte: NotRequired[int]
    """
    Return results where the `created` field is less than or equal to this value.
    """



import stripe


TEST_RESOURCE_ID = "cs_123"


class TestSession(object):
    def test_is_creatable(self, http_client_mock):
        resource = stripe.checkout.Session.create(
            cancel_url="https://stripe.com/cancel",
            client_reference_id="1234",
            line_items=[
                {
                    "amount": 123,
                    "currency": "usd",
                    "description": "item 1",
                    "images": ["https://stripe.com/img1"],
                    "name": "name",
                    "quantity": 2,
                }
            ],
            payment_intent_data={"receipt_email": "test@stripe.com"},
            payment_method_types=["card"],
            success_url="https://stripe.com/success",
        )
        http_client_mock.assert_requested(
            "post",
            path="/v1/checkout/sessions",
            post_data="cancel_url=https://stripe.com/cancel&client_reference_id=1234&line_items[0][amount]=123&line_items[0][currency]=usd&line_items[0][description]=item+1&line_items[0][images][0]=https://stripe.com/img1&line_items[0][name]=name&line_items[0][quantity]=2&payment_intent_data[receipt_email]=test@stripe.com&payment_method_types[0]=card&success_url=https://stripe.com/success",
        )
        assert isinstance(resource, stripe.checkout.Session)

    def test_is_retrievable(self, http_client_mock):
        resource = stripe.checkout.Session.retrieve(TEST_RESOURCE_ID)
        http_client_mock.assert_requested(
            "get", path="/v1/checkout/sessions/%s" % TEST_RESOURCE_ID
        )
        assert isinstance(resource, stripe.checkout.Session)


class TestSessionLineItems(object):
    def test_is_listable(self, http_client_mock):
        resources = stripe.checkout.Session.list_line_items(TEST_RESOURCE_ID)
        http_client_mock.assert_requested(
            "get",
            path="/v1/checkout/sessions/%s/line_items" % TEST_RESOURCE_ID,
        )
        assert isinstance(resources.data, list)
        assert isinstance(resources.data[0], stripe.LineItem)



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from typing import List
from typing_extensions import Literal, NotRequired, TypedDict


class InvoicePaymentListParams(RequestOptions):
    created: NotRequired["InvoicePaymentListParamsCreated|int"]
    """
    Only return invoice payments that were created during the given date interval.
    """
    ending_before: NotRequired[str]
    """
    A cursor for use in pagination. `ending_before` is an object ID that defines your place in the list. For instance, if you make a list request and receive 100 objects, starting with `obj_bar`, your subsequent call can include `ending_before=obj_bar` in order to fetch the previous page of the list.
    """
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    invoice: NotRequired[str]
    """
    The identifier of the invoice whose payments to return.
    """
    limit: NotRequired[int]
    """
    A limit on the number of objects to be returned. Limit can range between 1 and 100, and the default is 10.
    """
    payment: NotRequired["InvoicePaymentListParamsPayment"]
    """
    The payment details of the invoice payments to return.
    """
    starting_after: NotRequired[str]
    """
    A cursor for use in pagination. `starting_after` is an object ID that defines your place in the list. For instance, if you make a list request and receive 100 objects, ending with `obj_foo`, your subsequent call can include `starting_after=obj_foo` in order to fetch the next page of the list.
    """
    status: NotRequired[Literal["canceled", "open", "paid"]]
    """
    The status of the invoice payments to return.
    """


class InvoicePaymentListParamsCreated(TypedDict):
    gt: NotRequired[int]
    """
    Minimum value to filter by (exclusive)
    """
    gte: NotRequired[int]
    """
    Minimum value to filter by (inclusive)
    """
    lt: NotRequired[int]
    """
    Maximum value to filter by (exclusive)
    """
    lte: NotRequired[int]
    """
    Maximum value to filter by (inclusive)
    """


class InvoicePaymentListParamsPayment(TypedDict):
    payment_intent: NotRequired[str]
    """
    Only return invoice payments associated by this payment intent ID.
    """
    payment_record: NotRequired[str]
    """
    Only return invoice payments associated by this payment record ID.
    """
    type: Literal["payment_intent", "payment_record"]
    """
    Only return invoice payments associated by this payment type.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._request_options import RequestOptions
    from stripe.params.v2.reporting._report_run_create_params import (
        ReportRunCreateParams,
    )
    from stripe.params.v2.reporting._report_run_retrieve_params import (
        ReportRunRetrieveParams,
    )
    from stripe.v2.reporting._report_run import ReportRun


class ReportRunService(StripeService):
    def create(
        self,
        params: "ReportRunCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "ReportRun":
        """
        Initiates the generation of a `ReportRun` based on the specified report template
        and user-provided parameters. It's the starting point for obtaining report data,
        and returns a `ReportRun` object which can be used to track the progress and retrieve
        the results of the report.
        """
        return cast(
            "ReportRun",
            self._request(
                "post",
                "/v2/reporting/report_runs",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self,
        params: "ReportRunCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "ReportRun":
        """
        Initiates the generation of a `ReportRun` based on the specified report template
        and user-provided parameters. It's the starting point for obtaining report data,
        and returns a `ReportRun` object which can be used to track the progress and retrieve
        the results of the report.
        """
        return cast(
            "ReportRun",
            await self._request_async(
                "post",
                "/v2/reporting/report_runs",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        id: str,
        params: Optional["ReportRunRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ReportRun":
        """
        Fetches the current state and details of a previously created `ReportRun`. If the `ReportRun`
        has succeeded, the endpoint will provide details for how to retrieve the results.
        """
        return cast(
            "ReportRun",
            self._request(
                "get",
                "/v2/reporting/report_runs/{id}".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        id: str,
        params: Optional["ReportRunRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ReportRun":
        """
        Fetches the current state and details of a previously created `ReportRun`. If the `ReportRun`
        has succeeded, the endpoint will provide details for how to retrieve the results.
        """
        return cast(
            "ReportRun",
            await self._request_async(
                "get",
                "/v2/reporting/report_runs/{id}".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_object import StripeObject
from typing import ClassVar, Optional
from typing_extensions import Literal


class FinancialAddress(StripeObject):
    """
    A FinancialAddress contains information needed to transfer money to a Financial Account. A Financial Account can have more than one Financial Address.
    """

    OBJECT_NAME: ClassVar[Literal["v2.money_management.financial_address"]] = (
        "v2.money_management.financial_address"
    )

    class Credentials(StripeObject):
        class GbBankAccount(StripeObject):
            account_holder_name: str
            """
            The account holder name to be used during bank transference.
            """
            account_number: Optional[str]
            """
            The account number of the UK Bank Account.
            """
            last4: str
            """
            The last four digits of the UK Bank Account number. This will always be returned.
            To view the full account number when retrieving or listing FinancialAddresses, use the `include` request parameter.
            """
            sort_code: str
            """
            The sort code of the UK Bank Account.
            """

        class SepaBankAccount(StripeObject):
            account_holder_name: str
            """
            The account holder name to be used during bank transfers.
            """
            bank_name: str
            """
            The name of the Bank.
            """
            bic: str
            """
            The BIC of the SEPA Bank Account.
            """
            country: str
            """
            The originating country of the SEPA Bank account.
            """
            iban: str
            """
            The IBAN of the SEPA Bank Account.
            """
            last4: str
            """
            The last four digits of the SEPA Bank Account number. This will always be returned.
            To view the full account number when retrieving or listing FinancialAddresses, use the `include` request parameter.
            """

        class UsBankAccount(StripeObject):
            account_number: Optional[str]
            """
            The account number of the US Bank Account.
            """
            bank_name: Optional[str]
            """
            The name of the Bank.
            """
            last4: str
            """
            The last four digits of the US Bank Account number. This will always be returned.
            To view the full account number when retrieving or listing FinancialAddresses, use the `include` request parameter.
            """
            routing_number: str
            """
            The routing number of the US Bank Account.
            """
            swift_code: Optional[str]
            """
            The swift code of the bank or financial institution.
            """

        gb_bank_account: Optional[GbBankAccount]
        """
        The credentials of the UK Bank Account for the FinancialAddress. This contains unique banking details such as the sort code, account number, etc. of a UK bank account.
        """
        sepa_bank_account: Optional[SepaBankAccount]
        """
        The credentials of the SEPA Bank Account for the FinancialAddress. This contains unique banking details such as the IBAN, BIC, etc. of a SEPA bank account.
        """
        type: Literal[
            "gb_bank_account", "sepa_bank_account", "us_bank_account"
        ]
        """
        Open Enum. The type of Credentials that are provisioned for the FinancialAddress.
        """
        us_bank_account: Optional[UsBankAccount]
        """
        The credentials of the US Bank Account for the FinancialAddress. This contains unique banking details such as the routing number, account number, etc. of a US bank account.
        """
        _inner_class_types = {
            "gb_bank_account": GbBankAccount,
            "sepa_bank_account": SepaBankAccount,
            "us_bank_account": UsBankAccount,
        }

    created: str
    """
    The creation timestamp of the FinancialAddress.
    """
    credentials: Optional[Credentials]
    """
    Object indicates the type of credentials that have been allocated and attached to the FinancialAddress.
    It contains all necessary banking details with which to perform money movements with the FinancialAddress.
    This field is only available for FinancialAddresses with an active status.
    """
    currency: str
    """
    Open Enum. The currency the FinancialAddress supports.
    """
    financial_account: str
    """
    A ID of the FinancialAccount this FinancialAddress corresponds to.
    """
    id: str
    """
    The ID of a FinancialAddress.
    """
    livemode: bool
    """
    Has the value `true` if the object exists in live mode or the value `false` if the object exists in test mode.
    """
    object: Literal["v2.money_management.financial_address"]
    """
    String representing the object's type. Objects of the same type share the same value of the object field.
    """
    settlement_currency: Optional[str]
    """
    Open Enum. The currency the FinancialAddress settles into the FinancialAccount.
    """
    status: Literal["active", "archived", "failed", "pending"]
    """
    Closed Enum. An enum representing the status of the FinancialAddress. This indicates whether or not the FinancialAddress can be used for any money movement flows.
    """
    _inner_class_types = {"credentials": Credentials}



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from typing import List
from typing_extensions import Literal, NotRequired, TypedDict


class CadenceCancelParams(TypedDict):
    include: NotRequired[
        List[Literal["invoice_discount_rules", "settings_data"]]
    ]
    """
    Additional resource to include in the response.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._country_spec import CountrySpec
    from stripe._list_object import ListObject
    from stripe._request_options import RequestOptions
    from stripe.params._country_spec_list_params import CountrySpecListParams
    from stripe.params._country_spec_retrieve_params import (
        CountrySpecRetrieveParams,
    )


class CountrySpecService(StripeService):
    def list(
        self,
        params: Optional["CountrySpecListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[CountrySpec]":
        """
        Lists all Country Spec objects available in the API.
        """
        return cast(
            "ListObject[CountrySpec]",
            self._request(
                "get",
                "/v1/country_specs",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        params: Optional["CountrySpecListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[CountrySpec]":
        """
        Lists all Country Spec objects available in the API.
        """
        return cast(
            "ListObject[CountrySpec]",
            await self._request_async(
                "get",
                "/v1/country_specs",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        country: str,
        params: Optional["CountrySpecRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "CountrySpec":
        """
        Returns a Country Spec for a given Country code.
        """
        return cast(
            "CountrySpec",
            self._request(
                "get",
                "/v1/country_specs/{country}".format(
                    country=sanitize_id(country),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        country: str,
        params: Optional["CountrySpecRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "CountrySpec":
        """
        Returns a Country Spec for a given Country code.
        """
        return cast(
            "CountrySpec",
            await self._request_async(
                "get",
                "/v1/country_specs/{country}".format(
                    country=sanitize_id(country),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from typing import Dict, List, Union
from typing_extensions import Literal, NotRequired, TypedDict


class ExternalAccountCreateParams(TypedDict):
    default_for_currency: NotRequired[bool]
    """
    When set to true, or if this is the first external account added in this currency, this account becomes the default external account for its currency.
    """
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    external_account: Union[
        str,
        "ExternalAccountCreateParamsCard",
        "ExternalAccountCreateParamsBankAccount",
        "ExternalAccountCreateParamsCardToken",
    ]
    """
    Either a token, like the ones returned by [Stripe.js](https://stripe.com/docs/js), or a dictionary containing a user's external account details (with the options shown below).
    """
    metadata: NotRequired[Dict[str, str]]
    """
    Set of [key-value pairs](https://stripe.com/docs/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format. Individual keys can be unset by posting an empty value to them. All keys can be unset by posting an empty value to `metadata`.
    """


class ExternalAccountCreateParamsCard(TypedDict):
    object: Literal["card"]
    address_city: NotRequired[str]
    address_country: NotRequired[str]
    address_line1: NotRequired[str]
    address_line2: NotRequired[str]
    address_state: NotRequired[str]
    address_zip: NotRequired[str]
    currency: NotRequired[str]
    cvc: NotRequired[str]
    exp_month: int
    exp_year: int
    name: NotRequired[str]
    number: str
    metadata: NotRequired[Dict[str, str]]
    """
    Set of [key-value pairs](https://stripe.com/docs/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format.
    """


class ExternalAccountCreateParamsBankAccount(TypedDict):
    object: Literal["bank_account"]
    account_holder_name: NotRequired[str]
    """
    The name of the person or business that owns the bank account.This field is required when attaching the bank account to a `Customer` object.
    """
    account_holder_type: NotRequired[Literal["company", "individual"]]
    """
    The type of entity that holds the account. It can be `company` or `individual`. This field is required when attaching the bank account to a `Customer` object.
    """
    account_number: str
    """
    The account number for the bank account, in string form. Must be a checking account.
    """
    country: str
    """
    The country in which the bank account is located.
    """
    currency: NotRequired[str]
    """
    The currency the bank account is in. This must be a country/currency pairing that [Stripe supports.](docs/payouts)
    """
    routing_number: NotRequired[str]
    """
    The routing number, sort code, or other country-appropriate institution number for the bank account. For US bank accounts, this is required and should be the ACH routing number, not the wire routing number. If you are providing an IBAN for `account_number`, this field is not required.
    """


class ExternalAccountCreateParamsCardToken(TypedDict):
    object: Literal["card"]
    currency: NotRequired[str]
    token: str
